// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: mgard.proto
// Protobuf C++ Version: 7.36.0

#include "mgard.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::mgard::pb::VersionNumber
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::CartesianGridTopology
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::ExplicitCubeGeometry
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Domain
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Dataset
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::ErrorControl
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::DomainDecomposition
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::FunctionDecomposition
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Quantization
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::BitplaneEncoding
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Encoding
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Device
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
        // ::mgard::pb::Header
        {&::_pbi::kDescriptorMethods, &::descriptor_table_mgard_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace mgard {
namespace pb {
class VersionNumber::_Internal {
 public:
  using HasBits = decltype(::std::declval<VersionNumber>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_._has_bits_);
};

constexpr VersionNumber::ParseTableT_ VersionNumber::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::VersionNumber>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // uint64 major_ = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(VersionNumber, _impl_.major__), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.major__)}},
      // uint64 minor_ = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(VersionNumber, _impl_.minor__), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.minor__)}},
      // uint64 patch_ = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(VersionNumber, _impl_.patch__), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.patch__)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint64 major_ = 1;
      {PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.major__), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 minor_ = 2;
      {PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.minor__), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 patch_ = 3;
      {PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.patch__), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr VersionNumber::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : major__{::uint64_t{0u}},
        minor__{::uint64_t{0u}},
        patch__{::uint64_t{0u}} {}

template <typename>
constexpr VersionNumber::VersionNumber(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL VersionNumber::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) VersionNumber(arena);
}
constexpr auto VersionNumber::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(VersionNumber), alignof(VersionNumber));
}
constexpr auto VersionNumber::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &VersionNumber::MergeImpl,
          Super_::GetNewImpl<VersionNumber>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &VersionNumber::SharedDtor,
          Super_::GetClearImpl<VersionNumber>(), &VersionNumber::ByteSizeLong,
              &VersionNumber::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct VersionNumberGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr VersionNumberGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 VersionNumber_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(VersionNumber::InternalGenerateClassData_(
            _default, &VersionNumber_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<VersionNumber>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~VersionNumberGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) VersionNumber _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<VersionNumber>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(VersionNumberGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST VersionNumberGlobalsTypeInternal VersionNumber_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* VersionNumber_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return VersionNumber_globals_.GetClassData();
#else
  return VersionNumber_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Quantization::_Internal {
 public:
  using HasBits = decltype(::std::declval<Quantization>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Quantization, _impl_._has_bits_);
};

constexpr Quantization::ParseTableT_ Quantization::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Quantization, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Quantization>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool big_endian = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Quantization, _impl_.big_endian_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Quantization, _impl_.big_endian_)}},
      // .mgard.pb.Quantization.Method method = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Quantization, _impl_.method_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Quantization, _impl_.method_)}},
      // .mgard.pb.Quantization.BinWidths bin_widths = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Quantization, _impl_.bin_widths_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Quantization, _impl_.bin_widths_)}},
      // .mgard.pb.Quantization.Type type = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Quantization, _impl_.type_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Quantization, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.Quantization.Method method = 1;
      {PROTOBUF_FIELD_OFFSET(Quantization, _impl_.method_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.Quantization.BinWidths bin_widths = 2;
      {PROTOBUF_FIELD_OFFSET(Quantization, _impl_.bin_widths_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.Quantization.Type type = 3;
      {PROTOBUF_FIELD_OFFSET(Quantization, _impl_.type_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // bool big_endian = 4;
      {PROTOBUF_FIELD_OFFSET(Quantization, _impl_.big_endian_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Quantization::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : method_{static_cast< ::mgard::pb::Quantization_Method >(0)},
        bin_widths_{static_cast< ::mgard::pb::Quantization_BinWidths >(0)},
        type_{static_cast< ::mgard::pb::Quantization_Type >(0)},
        big_endian_{false} {}

template <typename>
constexpr Quantization::Quantization(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Quantization::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Quantization(arena);
}
constexpr auto Quantization::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Quantization), alignof(Quantization));
}
constexpr auto Quantization::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Quantization::MergeImpl,
          Super_::GetNewImpl<Quantization>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Quantization::SharedDtor,
          Super_::GetClearImpl<Quantization>(), &Quantization::ByteSizeLong,
              &Quantization::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Quantization, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct QuantizationGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr QuantizationGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Quantization_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Quantization::InternalGenerateClassData_(
            _default, &Quantization_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Quantization>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~QuantizationGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Quantization _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Quantization>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(QuantizationGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST QuantizationGlobalsTypeInternal Quantization_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Quantization_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Quantization_globals_.GetClassData();
#else
  return Quantization_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class FunctionDecomposition::_Internal {
 public:
  using HasBits = decltype(::std::declval<FunctionDecomposition>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_._has_bits_);
};

constexpr FunctionDecomposition::ParseTableT_ FunctionDecomposition::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::FunctionDecomposition>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .mgard.pb.FunctionDecomposition.Transform transform = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(FunctionDecomposition, _impl_.transform_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.transform_)}},
      // .mgard.pb.FunctionDecomposition.Hierarchy hierarchy = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(FunctionDecomposition, _impl_.hierarchy_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.hierarchy_)}},
      // uint64 L_target = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(FunctionDecomposition, _impl_.l_target_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.l_target_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.FunctionDecomposition.Transform transform = 1;
      {PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.transform_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.FunctionDecomposition.Hierarchy hierarchy = 2;
      {PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.hierarchy_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // uint64 L_target = 3;
      {PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.l_target_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr FunctionDecomposition::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : transform_{static_cast< ::mgard::pb::FunctionDecomposition_Transform >(0)},
        hierarchy_{static_cast< ::mgard::pb::FunctionDecomposition_Hierarchy >(0)},
        l_target_{::uint64_t{0u}} {}

template <typename>
constexpr FunctionDecomposition::FunctionDecomposition(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL FunctionDecomposition::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) FunctionDecomposition(arena);
}
constexpr auto FunctionDecomposition::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(FunctionDecomposition), alignof(FunctionDecomposition));
}
constexpr auto FunctionDecomposition::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &FunctionDecomposition::MergeImpl,
          Super_::GetNewImpl<FunctionDecomposition>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &FunctionDecomposition::SharedDtor,
          Super_::GetClearImpl<FunctionDecomposition>(), &FunctionDecomposition::ByteSizeLong,
              &FunctionDecomposition::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FunctionDecompositionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FunctionDecompositionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 FunctionDecomposition_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(FunctionDecomposition::InternalGenerateClassData_(
            _default, &FunctionDecomposition_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<FunctionDecomposition>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FunctionDecompositionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) FunctionDecomposition _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<FunctionDecomposition>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FunctionDecompositionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FunctionDecompositionGlobalsTypeInternal FunctionDecomposition_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* FunctionDecomposition_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return FunctionDecomposition_globals_.GetClassData();
#else
  return FunctionDecomposition_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExplicitCubeGeometry::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExplicitCubeGeometry>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometry, _impl_._has_bits_);
};

constexpr ExplicitCubeGeometry::ParseTableT_ ExplicitCubeGeometry::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometry, _impl_._has_bits_),
      0, // no _extensions_
      2, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967293,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::ExplicitCubeGeometry>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated double coordinates = 2;
      {::_pbi::TcParser::FastF64P1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometry, _impl_.coordinates_)}},
    }}, {{
      65535, 65535
    }}, {{
      // repeated double coordinates = 2;
      {PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometry, _impl_.coordinates_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedDouble)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ExplicitCubeGeometry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : coordinates_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mgard::pb::ExplicitCubeGeometry,
            PROTOBUF_FIELD_OFFSET(::mgard::pb::ExplicitCubeGeometry, _impl_.coordinates_)>()
         }
     {}

template <typename>
constexpr ExplicitCubeGeometry::ExplicitCubeGeometry(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExplicitCubeGeometry::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExplicitCubeGeometry(arena);
}
constexpr auto ExplicitCubeGeometry::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExplicitCubeGeometry), alignof(ExplicitCubeGeometry));
}
constexpr auto ExplicitCubeGeometry::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExplicitCubeGeometry::MergeImpl,
          Super_::GetNewImpl<ExplicitCubeGeometry>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExplicitCubeGeometry::SharedDtor,
          Super_::GetClearImpl<ExplicitCubeGeometry>(), &ExplicitCubeGeometry::ByteSizeLong,
              &ExplicitCubeGeometry::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometry, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExplicitCubeGeometryGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExplicitCubeGeometryGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExplicitCubeGeometry_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExplicitCubeGeometry::InternalGenerateClassData_(
            _default, &ExplicitCubeGeometry_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExplicitCubeGeometry>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExplicitCubeGeometryGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExplicitCubeGeometry _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExplicitCubeGeometry>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExplicitCubeGeometryGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExplicitCubeGeometryGlobalsTypeInternal ExplicitCubeGeometry_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExplicitCubeGeometry_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExplicitCubeGeometry_globals_.GetClassData();
#else
  return ExplicitCubeGeometry_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ErrorControl::_Internal {
 public:
  using HasBits = decltype(::std::declval<ErrorControl>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_._has_bits_);
};

constexpr ErrorControl::ParseTableT_ ErrorControl::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::ErrorControl>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .mgard.pb.ErrorControl.Mode mode = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ErrorControl, _impl_.mode_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.mode_)}},
      // .mgard.pb.ErrorControl.Norm norm = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ErrorControl, _impl_.norm_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.norm_)}},
      // double s = 3;
      {::_pbi::TcParser::FastF64S1,
       {25, 2, 0,
        PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.s_)}},
      // double norm_of_original_data = 4;
      {::_pbi::TcParser::FastF64S1,
       {33, 3, 0,
        PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.norm_of_original_data_)}},
      // double tolerance = 5;
      {::_pbi::TcParser::FastF64S1,
       {41, 4, 0,
        PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.tolerance_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.ErrorControl.Mode mode = 1;
      {PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.mode_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.ErrorControl.Norm norm = 2;
      {PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.norm_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // double s = 3;
      {PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.s_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // double norm_of_original_data = 4;
      {PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.norm_of_original_data_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
      // double tolerance = 5;
      {PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.tolerance_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kDouble)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr ErrorControl::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : mode_{static_cast< ::mgard::pb::ErrorControl_Mode >(0)},
        norm_{static_cast< ::mgard::pb::ErrorControl_Norm >(0)},
        s_{0},
        norm_of_original_data_{0},
        tolerance_{0} {}

template <typename>
constexpr ErrorControl::ErrorControl(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ErrorControl::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ErrorControl(arena);
}
constexpr auto ErrorControl::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ErrorControl), alignof(ErrorControl));
}
constexpr auto ErrorControl::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ErrorControl::MergeImpl,
          Super_::GetNewImpl<ErrorControl>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ErrorControl::SharedDtor,
          Super_::GetClearImpl<ErrorControl>(), &ErrorControl::ByteSizeLong,
              &ErrorControl::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ErrorControlGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ErrorControlGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ErrorControl_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ErrorControl::InternalGenerateClassData_(
            _default, &ErrorControl_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ErrorControl>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ErrorControlGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ErrorControl _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ErrorControl>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ErrorControlGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ErrorControlGlobalsTypeInternal ErrorControl_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ErrorControl_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ErrorControl_globals_.GetClassData();
#else
  return ErrorControl_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Encoding::_Internal {
 public:
  using HasBits = decltype(::std::declval<Encoding>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Encoding, _impl_._has_bits_);
};

constexpr Encoding::ParseTableT_ Encoding::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Encoding, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Encoding>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint64 huffman_block_size = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Encoding, _impl_.huffman_block_size_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Encoding, _impl_.huffman_block_size_)}},
      // .mgard.pb.Encoding.Preprocessor preprocessor = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Encoding, _impl_.preprocessor_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Encoding, _impl_.preprocessor_)}},
      // .mgard.pb.Encoding.Compressor compressor = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Encoding, _impl_.compressor_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Encoding, _impl_.compressor_)}},
      // uint64 huffman_dictionary_size = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Encoding, _impl_.huffman_dictionary_size_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Encoding, _impl_.huffman_dictionary_size_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.Encoding.Preprocessor preprocessor = 1;
      {PROTOBUF_FIELD_OFFSET(Encoding, _impl_.preprocessor_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.Encoding.Compressor compressor = 2;
      {PROTOBUF_FIELD_OFFSET(Encoding, _impl_.compressor_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // uint64 huffman_dictionary_size = 3;
      {PROTOBUF_FIELD_OFFSET(Encoding, _impl_.huffman_dictionary_size_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 huffman_block_size = 4;
      {PROTOBUF_FIELD_OFFSET(Encoding, _impl_.huffman_block_size_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Encoding::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : preprocessor_{static_cast< ::mgard::pb::Encoding_Preprocessor >(0)},
        compressor_{static_cast< ::mgard::pb::Encoding_Compressor >(0)},
        huffman_dictionary_size_{::uint64_t{0u}},
        huffman_block_size_{::uint64_t{0u}} {}

template <typename>
constexpr Encoding::Encoding(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Encoding::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Encoding(arena);
}
constexpr auto Encoding::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Encoding), alignof(Encoding));
}
constexpr auto Encoding::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Encoding::MergeImpl,
          Super_::GetNewImpl<Encoding>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Encoding::SharedDtor,
          Super_::GetClearImpl<Encoding>(), &Encoding::ByteSizeLong,
              &Encoding::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Encoding, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct EncodingGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr EncodingGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Encoding_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Encoding::InternalGenerateClassData_(
            _default, &Encoding_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Encoding>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~EncodingGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Encoding _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Encoding>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(EncodingGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST EncodingGlobalsTypeInternal Encoding_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Encoding_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Encoding_globals_.GetClassData();
#else
  return Encoding_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class DomainDecomposition::_Internal {
 public:
  using HasBits = decltype(::std::declval<DomainDecomposition>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_._has_bits_);
};

constexpr DomainDecomposition::ParseTableT_ DomainDecomposition::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::DomainDecomposition>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .mgard.pb.DomainDecomposition.Method method = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(DomainDecomposition, _impl_.method_), 2>(),
       {8, 2, 0,
        PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.method_)}},
      // uint64 decomposition_dimension = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(DomainDecomposition, _impl_.decomposition_dimension_), 0>(),
       {16, 0, 0,
        PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.decomposition_dimension_)}},
      // uint64 decomposition_size = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(DomainDecomposition, _impl_.decomposition_size_), 1>(),
       {24, 1, 0,
        PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.decomposition_size_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.DomainDecomposition.Method method = 1;
      {PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.method_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // uint64 decomposition_dimension = 2;
      {PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.decomposition_dimension_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 decomposition_size = 3;
      {PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.decomposition_size_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr DomainDecomposition::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : decomposition_dimension_{::uint64_t{0u}},
        decomposition_size_{::uint64_t{0u}},
        method_{static_cast< ::mgard::pb::DomainDecomposition_Method >(0)} {}

template <typename>
constexpr DomainDecomposition::DomainDecomposition(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL DomainDecomposition::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) DomainDecomposition(arena);
}
constexpr auto DomainDecomposition::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(DomainDecomposition), alignof(DomainDecomposition));
}
constexpr auto DomainDecomposition::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &DomainDecomposition::MergeImpl,
          Super_::GetNewImpl<DomainDecomposition>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &DomainDecomposition::SharedDtor,
          Super_::GetClearImpl<DomainDecomposition>(), &DomainDecomposition::ByteSizeLong,
              &DomainDecomposition::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DomainDecompositionGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DomainDecompositionGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 DomainDecomposition_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(DomainDecomposition::InternalGenerateClassData_(
            _default, &DomainDecomposition_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<DomainDecomposition>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DomainDecompositionGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) DomainDecomposition _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<DomainDecomposition>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DomainDecompositionGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DomainDecompositionGlobalsTypeInternal DomainDecomposition_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* DomainDecomposition_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return DomainDecomposition_globals_.GetClassData();
#else
  return DomainDecomposition_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Device::_Internal {
 public:
  using HasBits = decltype(::std::declval<Device>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Device, _impl_._has_bits_);
};

constexpr Device::ParseTableT_ Device::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Device, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Device>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .mgard.pb.Device.Backend backend = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Device, _impl_.backend_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Device, _impl_.backend_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.Device.Backend backend = 1;
      {PROTOBUF_FIELD_OFFSET(Device, _impl_.backend_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Device::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : backend_{static_cast< ::mgard::pb::Device_Backend >(0)} {}

template <typename>
constexpr Device::Device(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Device::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Device(arena);
}
constexpr auto Device::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Device), alignof(Device));
}
constexpr auto Device::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Device::MergeImpl,
          Super_::GetNewImpl<Device>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Device::SharedDtor,
          Super_::GetClearImpl<Device>(), &Device::ByteSizeLong,
              &Device::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Device, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DeviceGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DeviceGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Device_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Device::InternalGenerateClassData_(
            _default, &Device_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Device>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DeviceGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Device _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Device>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DeviceGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DeviceGlobalsTypeInternal Device_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Device_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Device_globals_.GetClassData();
#else
  return Device_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Dataset::_Internal {
 public:
  using HasBits = decltype(::std::declval<Dataset>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Dataset, _impl_._has_bits_);
};

constexpr Dataset::ParseTableT_ Dataset::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Dataset, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Dataset>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint64 dimension = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Dataset, _impl_.dimension_), 0>(),
       {16, 0, 0,
        PROTOBUF_FIELD_OFFSET(Dataset, _impl_.dimension_)}},
      // .mgard.pb.Dataset.Type type = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Dataset, _impl_.type_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(Dataset, _impl_.type_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.Dataset.Type type = 1;
      {PROTOBUF_FIELD_OFFSET(Dataset, _impl_.type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // uint64 dimension = 2;
      {PROTOBUF_FIELD_OFFSET(Dataset, _impl_.dimension_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Dataset::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : dimension_{::uint64_t{0u}},
        type_{static_cast< ::mgard::pb::Dataset_Type >(0)} {}

template <typename>
constexpr Dataset::Dataset(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Dataset::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Dataset(arena);
}
constexpr auto Dataset::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Dataset), alignof(Dataset));
}
constexpr auto Dataset::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Dataset::MergeImpl,
          Super_::GetNewImpl<Dataset>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Dataset::SharedDtor,
          Super_::GetClearImpl<Dataset>(), &Dataset::ByteSizeLong,
              &Dataset::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Dataset, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DatasetGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DatasetGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Dataset_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Dataset::InternalGenerateClassData_(
            _default, &Dataset_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Dataset>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DatasetGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Dataset _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Dataset>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DatasetGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DatasetGlobalsTypeInternal Dataset_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Dataset_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Dataset_globals_.GetClassData();
#else
  return Dataset_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CartesianGridTopology::_Internal {
 public:
  using HasBits = decltype(::std::declval<CartesianGridTopology>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_._has_bits_);
};

constexpr CartesianGridTopology::ParseTableT_ CartesianGridTopology::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::CartesianGridTopology>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // repeated uint64 shape = 2;
      {::_pbi::TcParser::FastV64P1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_.shape_)}},
      // uint64 dimension = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(CartesianGridTopology, _impl_.dimension_), 1>(),
       {8, 1, 0,
        PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_.dimension_)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint64 dimension = 1;
      {PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_.dimension_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // repeated uint64 shape = 2;
      {PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_.shape_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr CartesianGridTopology::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : shape_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mgard::pb::CartesianGridTopology,
            PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_.shape_)>()
         }
        ,
        dimension_{::uint64_t{0u}} {}

template <typename>
constexpr CartesianGridTopology::CartesianGridTopology(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CartesianGridTopology::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CartesianGridTopology(arena);
}
constexpr auto CartesianGridTopology::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(CartesianGridTopology), alignof(CartesianGridTopology));
}
constexpr auto CartesianGridTopology::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CartesianGridTopology::MergeImpl,
          Super_::GetNewImpl<CartesianGridTopology>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CartesianGridTopology::SharedDtor,
          Super_::GetClearImpl<CartesianGridTopology>(), &CartesianGridTopology::ByteSizeLong,
              &CartesianGridTopology::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CartesianGridTopology, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CartesianGridTopologyGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CartesianGridTopologyGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CartesianGridTopology_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CartesianGridTopology::InternalGenerateClassData_(
            _default, &CartesianGridTopology_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CartesianGridTopology>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CartesianGridTopologyGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CartesianGridTopology _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CartesianGridTopology>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CartesianGridTopologyGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CartesianGridTopologyGlobalsTypeInternal CartesianGridTopology_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CartesianGridTopology_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CartesianGridTopology_globals_.GetClassData();
#else
  return CartesianGridTopology_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class BitplaneEncoding::_Internal {
 public:
  using HasBits = decltype(::std::declval<BitplaneEncoding>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_._has_bits_);
};

constexpr BitplaneEncoding::ParseTableT_ BitplaneEncoding::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::BitplaneEncoding>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bool big_endian = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(BitplaneEncoding, _impl_.big_endian_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.big_endian_)}},
      // .mgard.pb.BitplaneEncoding.Method method = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BitplaneEncoding, _impl_.method_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.method_)}},
      // .mgard.pb.BitplaneEncoding.Type type = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(BitplaneEncoding, _impl_.type_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.type_)}},
      // uint64 number_bitplanes = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(BitplaneEncoding, _impl_.number_bitplanes_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.number_bitplanes_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.BitplaneEncoding.Method method = 1;
      {PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.method_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.BitplaneEncoding.Type type = 2;
      {PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.type_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // uint64 number_bitplanes = 3;
      {PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.number_bitplanes_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // bool big_endian = 4;
      {PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.big_endian_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr BitplaneEncoding::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : method_{static_cast< ::mgard::pb::BitplaneEncoding_Method >(0)},
        type_{static_cast< ::mgard::pb::BitplaneEncoding_Type >(0)},
        number_bitplanes_{::uint64_t{0u}},
        big_endian_{false} {}

template <typename>
constexpr BitplaneEncoding::BitplaneEncoding(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL BitplaneEncoding::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) BitplaneEncoding(arena);
}
constexpr auto BitplaneEncoding::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(BitplaneEncoding), alignof(BitplaneEncoding));
}
constexpr auto BitplaneEncoding::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &BitplaneEncoding::MergeImpl,
          Super_::GetNewImpl<BitplaneEncoding>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &BitplaneEncoding::SharedDtor,
          Super_::GetClearImpl<BitplaneEncoding>(), &BitplaneEncoding::ByteSizeLong,
              &BitplaneEncoding::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BitplaneEncodingGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BitplaneEncodingGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 BitplaneEncoding_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(BitplaneEncoding::InternalGenerateClassData_(
            _default, &BitplaneEncoding_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<BitplaneEncoding>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BitplaneEncodingGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) BitplaneEncoding _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<BitplaneEncoding>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BitplaneEncodingGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BitplaneEncodingGlobalsTypeInternal BitplaneEncoding_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* BitplaneEncoding_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return BitplaneEncoding_globals_.GetClassData();
#else
  return BitplaneEncoding_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Domain::_Internal {
 public:
  using HasBits = decltype(::std::declval<Domain>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Domain, _impl_._has_bits_);
  static constexpr ::int32_t kOneofCaseOffset =
      PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_._oneof_case_);
};

constexpr Domain::ParseTableT_ Domain::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Domain, _impl_._has_bits_),
      0, // no _extensions_
      5, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Domain>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .mgard.pb.ExplicitCubeGeometry explicit_cube_geometry = 4;
      {::_pbi::TcParser::FastMiniParse1,
       {34, ::uint8_t{6},
        ::uint32_t{
            PROTOBUF_FIELD_OFFSET(ParseTableT_, field_entries) +
            sizeof(_pbi::TcParseTableBase::FieldEntry) * 3}}},
      // .mgard.pb.Domain.Topology topology = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Domain, _impl_.topology_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Domain, _impl_.topology_)}},
      // .mgard.pb.CartesianGridTopology cartesian_grid_topology = 2;
      {::_pbi::TcParser::FastMiniParse1,
       {18, ::uint8_t{6},
        ::uint32_t{
            PROTOBUF_FIELD_OFFSET(ParseTableT_, field_entries) +
            sizeof(_pbi::TcParseTableBase::FieldEntry) * 1}}},
      // .mgard.pb.Domain.Geometry geometry = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Domain, _impl_.geometry_), 1>(),
       {24, 1, 0,
        PROTOBUF_FIELD_OFFSET(Domain, _impl_.geometry_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.Domain.Topology topology = 1;
      {PROTOBUF_FIELD_OFFSET(Domain, _impl_.topology_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.CartesianGridTopology cartesian_grid_topology = 2;
      {PROTOBUF_FIELD_OFFSET(Domain, _impl_.topology_definition_.cartesian_grid_topology_), _Internal::kOneofCaseOffset + 0, 0, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Domain.Geometry geometry = 3;
      {PROTOBUF_FIELD_OFFSET(Domain, _impl_.geometry_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kOpenEnum)},
      // .mgard.pb.ExplicitCubeGeometry explicit_cube_geometry = 4;
      {PROTOBUF_FIELD_OFFSET(Domain, _impl_.geometry_definition_.explicit_cube_geometry_), _Internal::kOneofCaseOffset + 4, 1, (0 | ::_fl::kFcOneof | ::_fl::kMessage | ::_fl::kTvTable)},
      // string explicit_cube_filename = 5;
      {PROTOBUF_FIELD_OFFSET(Domain, _impl_.geometry_definition_.explicit_cube_filename_), _Internal::kOneofCaseOffset + 4, 0, (0 | ::_fl::kFcOneof | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::CartesianGridTopology>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::CartesianGridTopology_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::ExplicitCubeGeometry>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::ExplicitCubeGeometry_globals_},
        #endif
    }},
    {{
      "\17\0\0\0\0\26\0\0"
      "mgard.pb.Domain"
      "explicit_cube_filename"
    }},
  };
}


inline constexpr Domain::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : topology_{static_cast< ::mgard::pb::Domain_Topology >(0)},
        geometry_{static_cast< ::mgard::pb::Domain_Geometry >(0)},
        topology_definition_{},
        geometry_definition_{},
        _oneof_case_{} {}

template <typename>
constexpr Domain::Domain(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Domain::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Domain(arena);
}
constexpr auto Domain::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Domain), alignof(Domain));
}
constexpr auto Domain::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Domain::MergeImpl,
          Super_::GetNewImpl<Domain>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Domain::SharedDtor,
          Super_::GetClearImpl<Domain>(), &Domain::ByteSizeLong,
              &Domain::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Domain, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DomainGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DomainGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Domain_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Domain::InternalGenerateClassData_(
            _default, &Domain_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Domain>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DomainGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Domain _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Domain>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DomainGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DomainGlobalsTypeInternal Domain_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Domain_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Domain_globals_.GetClassData();
#else
  return Domain_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Header::_Internal {
 public:
  using HasBits = decltype(::std::declval<Header>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Header, _impl_._has_bits_);
};

constexpr Header::ParseTableT_ Header::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Header, _impl_._has_bits_),
      0, // no _extensions_
      12, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294963201,  // skipmap
      offsetof(ParseTableT_, field_entries),
      11,  // num_field_entries
      11,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::mgard::pb::Header>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      // .mgard.pb.VersionNumber mgard_version = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 0, 0,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.mgard_version_)}},
      // .mgard.pb.VersionNumber file_format_version = 3;
      {::_pbi::TcParser::FastMtS1,
       {26, 1, 1,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.file_format_version_)}},
      // .mgard.pb.Domain domain = 4;
      {::_pbi::TcParser::FastMtS1,
       {34, 2, 2,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.domain_)}},
      // .mgard.pb.Dataset dataset = 5;
      {::_pbi::TcParser::FastMtS1,
       {42, 3, 3,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.dataset_)}},
      // .mgard.pb.ErrorControl error_control = 6;
      {::_pbi::TcParser::FastMtS1,
       {50, 4, 4,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.error_control_)}},
      // .mgard.pb.DomainDecomposition domain_decomposition = 7;
      {::_pbi::TcParser::FastMtS1,
       {58, 5, 5,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.domain_decomposition_)}},
      // .mgard.pb.FunctionDecomposition function_decomposition = 8;
      {::_pbi::TcParser::FastMtS1,
       {66, 6, 6,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.function_decomposition_)}},
      // .mgard.pb.Quantization quantization = 9;
      {::_pbi::TcParser::FastMtS1,
       {74, 7, 7,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.quantization_)}},
      // .mgard.pb.BitplaneEncoding bitplane_encoding = 10;
      {::_pbi::TcParser::FastMtS1,
       {82, 8, 8,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.bitplane_encoding_)}},
      // .mgard.pb.Encoding encoding = 11;
      {::_pbi::TcParser::FastMtS1,
       {90, 9, 9,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.encoding_)}},
      // .mgard.pb.Device device = 12;
      {::_pbi::TcParser::FastMtS1,
       {98, 10, 10,
        PROTOBUF_FIELD_OFFSET(Header, _impl_.device_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .mgard.pb.VersionNumber mgard_version = 2;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.mgard_version_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.VersionNumber file_format_version = 3;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.file_format_version_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Domain domain = 4;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.domain_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Dataset dataset = 5;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.dataset_), _Internal::kHasBitsOffset + 3, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.ErrorControl error_control = 6;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.error_control_), _Internal::kHasBitsOffset + 4, 4, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.DomainDecomposition domain_decomposition = 7;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.domain_decomposition_), _Internal::kHasBitsOffset + 5, 5, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.FunctionDecomposition function_decomposition = 8;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.function_decomposition_), _Internal::kHasBitsOffset + 6, 6, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Quantization quantization = 9;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.quantization_), _Internal::kHasBitsOffset + 7, 7, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.BitplaneEncoding bitplane_encoding = 10;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.bitplane_encoding_), _Internal::kHasBitsOffset + 8, 8, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Encoding encoding = 11;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.encoding_), _Internal::kHasBitsOffset + 9, 9, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .mgard.pb.Device device = 12;
      {PROTOBUF_FIELD_OFFSET(Header, _impl_.device_), _Internal::kHasBitsOffset + 10, 10, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::VersionNumber>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::VersionNumber_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::VersionNumber>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::VersionNumber_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::Domain>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::Domain_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::Dataset>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::Dataset_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::ErrorControl>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::ErrorControl_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::DomainDecomposition>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::DomainDecomposition_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::FunctionDecomposition>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::FunctionDecomposition_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::Quantization>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::Quantization_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::BitplaneEncoding>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::BitplaneEncoding_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::Encoding>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::Encoding_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::mgard::pb::Device>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::mgard::pb::Device_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Header::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : mgard_version_{nullptr},
        file_format_version_{nullptr},
        domain_{nullptr},
        dataset_{nullptr},
        error_control_{nullptr},
        domain_decomposition_{nullptr},
        function_decomposition_{nullptr},
        quantization_{nullptr},
        bitplane_encoding_{nullptr},
        encoding_{nullptr},
        device_{nullptr} {}

template <typename>
constexpr Header::Header(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Header::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Header(arena);
}
constexpr auto Header::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Header), alignof(Header));
}
constexpr auto Header::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Header::MergeImpl,
          Super_::GetNewImpl<Header>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Header::SharedDtor,
          Super_::GetClearImpl<Header>(), &Header::ByteSizeLong,
              &Header::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Header, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_mgard_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct HeaderGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HeaderGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Header_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Header::InternalGenerateClassData_(
            _default, &Header_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Header>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HeaderGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Header _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Header>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HeaderGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HeaderGlobalsTypeInternal Header_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Header_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Header_globals_.GetClassData();
#else
  return Header_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace pb
}  // namespace mgard
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_mgard_2eproto[16];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_mgard_2eproto = nullptr;
const ::uint32_t
    TableStruct_mgard_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::VersionNumber, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::VersionNumber, _impl_.major__),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::VersionNumber, _impl_.minor__),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::VersionNumber, _impl_.patch__),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_.dimension_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_.shape_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ExplicitCubeGeometry, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ExplicitCubeGeometry, _impl_.coordinates_),
        0,
        0x085, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_._has_bits_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_._oneof_case_[0]),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.topology_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.topology_definition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.geometry_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.geometry_definition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.geometry_definition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.topology_definition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Domain, _impl_.geometry_definition_),
        0,
        ~0u,
        1,
        ~0u,
        ~0u,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Dataset, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Dataset, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Dataset, _impl_.dimension_),
        1,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_.mode_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_.norm_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_.s_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_.norm_of_original_data_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::ErrorControl, _impl_.tolerance_),
        0,
        1,
        2,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::DomainDecomposition, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::DomainDecomposition, _impl_.method_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::DomainDecomposition, _impl_.decomposition_dimension_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::DomainDecomposition, _impl_.decomposition_size_),
        2,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::FunctionDecomposition, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::FunctionDecomposition, _impl_.transform_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::FunctionDecomposition, _impl_.hierarchy_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::FunctionDecomposition, _impl_.l_target_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Quantization, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Quantization, _impl_.method_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Quantization, _impl_.bin_widths_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Quantization, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Quantization, _impl_.big_endian_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::BitplaneEncoding, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::BitplaneEncoding, _impl_.method_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::BitplaneEncoding, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::BitplaneEncoding, _impl_.number_bitplanes_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::BitplaneEncoding, _impl_.big_endian_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Encoding, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Encoding, _impl_.preprocessor_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Encoding, _impl_.compressor_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Encoding, _impl_.huffman_dictionary_size_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Encoding, _impl_.huffman_block_size_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Device, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Device, _impl_.backend_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_._has_bits_),
        14, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.mgard_version_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.file_format_version_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.domain_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.dataset_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.error_control_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.domain_decomposition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.function_decomposition_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.quantization_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.bitplane_encoding_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.encoding_),
        PROTOBUF_FIELD_OFFSET(::mgard::pb::Header, _impl_.device_),
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        10,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::mgard::pb::VersionNumber)},
        {9, sizeof(::mgard::pb::CartesianGridTopology)},
        {16, sizeof(::mgard::pb::ExplicitCubeGeometry)},
        {21, sizeof(::mgard::pb::Domain)},
        {37, sizeof(::mgard::pb::Dataset)},
        {44, sizeof(::mgard::pb::ErrorControl)},
        {57, sizeof(::mgard::pb::DomainDecomposition)},
        {66, sizeof(::mgard::pb::FunctionDecomposition)},
        {75, sizeof(::mgard::pb::Quantization)},
        {86, sizeof(::mgard::pb::BitplaneEncoding)},
        {97, sizeof(::mgard::pb::Encoding)},
        {108, sizeof(::mgard::pb::Device)},
        {113, sizeof(::mgard::pb::Header)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::mgard::pb::VersionNumber_globals_,
        &::mgard::pb::CartesianGridTopology_globals_,
        &::mgard::pb::ExplicitCubeGeometry_globals_,
        &::mgard::pb::Domain_globals_,
        &::mgard::pb::Dataset_globals_,
        &::mgard::pb::ErrorControl_globals_,
        &::mgard::pb::DomainDecomposition_globals_,
        &::mgard::pb::FunctionDecomposition_globals_,
        &::mgard::pb::Quantization_globals_,
        &::mgard::pb::BitplaneEncoding_globals_,
        &::mgard::pb::Encoding_globals_,
        &::mgard::pb::Device_globals_,
        &::mgard::pb::Header_globals_,
};
const char descriptor_table_protodef_mgard_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\013mgard.proto\022\010mgard.pb\"\?\n\rVersionNumber"
    "\022\016\n\006major_\030\001 \001(\004\022\016\n\006minor_\030\002 \001(\004\022\016\n\006patc"
    "h_\030\003 \001(\004\"9\n\025CartesianGridTopology\022\021\n\tdim"
    "ension\030\001 \001(\004\022\r\n\005shape\030\002 \003(\004\"+\n\024ExplicitC"
    "ubeGeometry\022\023\n\013coordinates\030\002 \003(\001\"\206\003\n\006Dom"
    "ain\022+\n\010topology\030\001 \001(\0162\031.mgard.pb.Domain."
    "Topology\022B\n\027cartesian_grid_topology\030\002 \001("
    "\0132\037.mgard.pb.CartesianGridTopologyH\000\022+\n\010"
    "geometry\030\003 \001(\0162\031.mgard.pb.Domain.Geometr"
    "y\022@\n\026explicit_cube_geometry\030\004 \001(\0132\036.mgar"
    "d.pb.ExplicitCubeGeometryH\001\022 \n\026explicit_"
    "cube_filename\030\005 \001(\tH\001\"\036\n\010Topology\022\022\n\016CAR"
    "TESIAN_GRID\020\000\",\n\010Geometry\022\r\n\tUNIT_CUBE\020\000"
    "\022\021\n\rEXPLICIT_CUBE\020\001B\025\n\023topology_definiti"
    "onB\025\n\023geometry_definition\"a\n\007Dataset\022$\n\004"
    "type\030\001 \001(\0162\026.mgard.pb.Dataset.Type\022\021\n\tdi"
    "mension\030\002 \001(\004\"\035\n\004Type\022\t\n\005FLOAT\020\000\022\n\n\006DOUB"
    "LE\020\001\"\351\001\n\014ErrorControl\022)\n\004mode\030\001 \001(\0162\033.mg"
    "ard.pb.ErrorControl.Mode\022)\n\004norm\030\002 \001(\0162\033"
    ".mgard.pb.ErrorControl.Norm\022\t\n\001s\030\003 \001(\001\022\035"
    "\n\025norm_of_original_data\030\004 \001(\001\022\021\n\ttoleran"
    "ce\030\005 \001(\001\"\"\n\004Mode\022\014\n\010ABSOLUTE\020\000\022\014\n\010RELATI"
    "VE\020\001\"\"\n\004Norm\022\016\n\nL_INFINITY\020\000\022\n\n\006S_NORM\020\001"
    "\"\317\001\n\023DomainDecomposition\0224\n\006method\030\001 \001(\016"
    "2$.mgard.pb.DomainDecomposition.Method\022\037"
    "\n\027decomposition_dimension\030\002 \001(\004\022\032\n\022decom"
    "position_size\030\003 \001(\004\"E\n\006Method\022\017\n\013NOOP_ME"
    "THOD\020\000\022\021\n\rMAX_DIMENSION\020\001\022\t\n\005BLOCK\020\002\022\014\n\010"
    "VARIABLE\020\003\"\333\002\n\025FunctionDecomposition\022<\n\t"
    "transform\030\001 \001(\0162).mgard.pb.FunctionDecom"
    "position.Transform\022<\n\thierarchy\030\002 \001(\0162)."
    "mgard.pb.FunctionDecomposition.Hierarchy"
    "\022\020\n\010L_target\030\003 \001(\004\"(\n\tTransform\022\033\n\027MULTI"
    "LEVEL_COEFFICIENTS\020\000\"\211\001\n\tHierarchy\022\031\n\025PO"
    "WER_OF_TWO_PLUS_ONE\020\000\022#\n\037MULTIDIMENSION_"
    "WITH_GHOST_NODES\020\001\022&\n\"ONE_DIM_AT_A_TIME_"
    "WITH_GHOST_NODES\020\002\022\024\n\020HYBRID_HIERARCHY\020\003"
    "\"\333\002\n\014Quantization\022-\n\006method\030\001 \001(\0162\035.mgar"
    "d.pb.Quantization.Method\0224\n\nbin_widths\030\002"
    " \001(\0162 .mgard.pb.Quantization.BinWidths\022)"
    "\n\004type\030\003 \001(\0162\033.mgard.pb.Quantization.Typ"
    "e\022\022\n\nbig_endian\030\004 \001(\010\";\n\006Method\022\025\n\021NOOP_"
    "QUANTIZATION\020\000\022\032\n\026COEFFICIENTWISE_LINEAR"
    "\020\001\"/\n\tBinWidths\022\023\n\017PER_COEFFICIENT\020\000\022\r\n\t"
    "PER_LEVEL\020\001\"9\n\004Type\022\n\n\006INT8_T\020\000\022\013\n\007INT16"
    "_T\020\001\022\013\n\007INT32_T\020\002\022\013\n\007INT64_T\020\003\"\242\002\n\020Bitpl"
    "aneEncoding\0221\n\006method\030\001 \001(\0162!.mgard.pb.B"
    "itplaneEncoding.Method\022-\n\004type\030\002 \001(\0162\037.m"
    "gard.pb.BitplaneEncoding.Type\022\030\n\020number_"
    "bitplanes\030\003 \001(\004\022\022\n\nbig_endian\030\004 \001(\010\"C\n\006M"
    "ethod\022\032\n\026NOOP_BITPLANE_ENCODING\020\000\022\035\n\031GRO"
    "UPED_BITPLANE_ENCODING\020\001\"9\n\004Type\022\n\n\006INT8"
    "_T\020\000\022\013\n\007INT16_T\020\001\022\013\n\007INT32_T\020\002\022\013\n\007INT64_"
    "T\020\003\"\353\002\n\010Encoding\0225\n\014preprocessor\030\001 \001(\0162\037"
    ".mgard.pb.Encoding.Preprocessor\0221\n\ncompr"
    "essor\030\002 \001(\0162\035.mgard.pb.Encoding.Compress"
    "or\022\037\n\027huffman_dictionary_size\030\003 \001(\004\022\032\n\022h"
    "uffman_block_size\030\004 \001(\004\"2\n\014Preprocessor\022"
    "\025\n\021NOOP_PREPROCESSOR\020\000\022\013\n\007SHUFFLE\020\001\"\203\001\n\n"
    "Compressor\022\023\n\017NOOP_COMPRESSOR\020\000\022\024\n\020CPU_H"
    "UFFMAN_ZLIB\020\001\022\024\n\020CPU_HUFFMAN_ZSTD\020\002\022\r\n\tX"
    "_HUFFMAN\020\003\022\021\n\rX_HUFFMAN_LZ4\020\004\022\022\n\016X_HUFFM"
    "AN_ZSTD\020\005\"\206\001\n\006Device\022)\n\007backend\030\001 \001(\0162\030."
    "mgard.pb.Device.Backend\"Q\n\007Backend\022\007\n\003CP"
    "U\020\000\022\014\n\010X_SERIAL\020\001\022\014\n\010X_OPENMP\020\002\022\n\n\006X_CUD"
    "A\020\003\022\t\n\005X_HIP\020\004\022\n\n\006X_SYCL\020\005\"\216\004\n\006Header\022.\n"
    "\rmgard_version\030\002 \001(\0132\027.mgard.pb.VersionN"
    "umber\0224\n\023file_format_version\030\003 \001(\0132\027.mga"
    "rd.pb.VersionNumber\022 \n\006domain\030\004 \001(\0132\020.mg"
    "ard.pb.Domain\022\"\n\007dataset\030\005 \001(\0132\021.mgard.p"
    "b.Dataset\022-\n\rerror_control\030\006 \001(\0132\026.mgard"
    ".pb.ErrorControl\022;\n\024domain_decomposition"
    "\030\007 \001(\0132\035.mgard.pb.DomainDecomposition\022\?\n"
    "\026function_decomposition\030\010 \001(\0132\037.mgard.pb"
    ".FunctionDecomposition\022,\n\014quantization\030\t"
    " \001(\0132\026.mgard.pb.Quantization\0225\n\021bitplane"
    "_encoding\030\n \001(\0132\032.mgard.pb.BitplaneEncod"
    "ing\022$\n\010encoding\030\013 \001(\0132\022.mgard.pb.Encodin"
    "g\022 \n\006device\030\014 \001(\0132\020.mgard.pb.Deviceb\006pro"
    "to3"
};
static ::absl::once_flag descriptor_table_mgard_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_mgard_2eproto = {
    false,
    false,
    3163,
    descriptor_table_protodef_mgard_2eproto,
    "mgard.proto",
    &descriptor_table_mgard_2eproto_once,
    nullptr,
    0,
    13,
    schemas,
    file_message_globals,
    TableStruct_mgard_2eproto::offsets,
    file_level_enum_descriptors_mgard_2eproto,
    file_level_service_descriptors_mgard_2eproto,
};
namespace mgard {
namespace pb {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Domain_Topology_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t Domain_Topology_internal_data_[] = {
    65536u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Domain_Geometry_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[1];
}
PROTOBUF_CONSTINIT const uint32_t Domain_Geometry_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Dataset_Type_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[2];
}
PROTOBUF_CONSTINIT const uint32_t Dataset_Type_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ErrorControl_Mode_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[3];
}
PROTOBUF_CONSTINIT const uint32_t ErrorControl_Mode_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
ErrorControl_Norm_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[4];
}
PROTOBUF_CONSTINIT const uint32_t ErrorControl_Norm_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
DomainDecomposition_Method_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[5];
}
PROTOBUF_CONSTINIT const uint32_t DomainDecomposition_Method_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
FunctionDecomposition_Transform_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[6];
}
PROTOBUF_CONSTINIT const uint32_t FunctionDecomposition_Transform_internal_data_[] = {
    65536u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
FunctionDecomposition_Hierarchy_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[7];
}
PROTOBUF_CONSTINIT const uint32_t FunctionDecomposition_Hierarchy_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Quantization_Method_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[8];
}
PROTOBUF_CONSTINIT const uint32_t Quantization_Method_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Quantization_BinWidths_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[9];
}
PROTOBUF_CONSTINIT const uint32_t Quantization_BinWidths_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Quantization_Type_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[10];
}
PROTOBUF_CONSTINIT const uint32_t Quantization_Type_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
BitplaneEncoding_Method_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[11];
}
PROTOBUF_CONSTINIT const uint32_t BitplaneEncoding_Method_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
BitplaneEncoding_Type_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[12];
}
PROTOBUF_CONSTINIT const uint32_t BitplaneEncoding_Type_internal_data_[] = {
    262144u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Encoding_Preprocessor_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[13];
}
PROTOBUF_CONSTINIT const uint32_t Encoding_Preprocessor_internal_data_[] = {
    131072u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Encoding_Compressor_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[14];
}
PROTOBUF_CONSTINIT const uint32_t Encoding_Compressor_internal_data_[] = {
    393216u, 0u, };
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
Device_Backend_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_mgard_2eproto);
  return file_level_enum_descriptors_mgard_2eproto[15];
}
PROTOBUF_CONSTINIT const uint32_t Device_Backend_internal_data_[] = {
    393216u, 0u, };
// ===================================================================

VersionNumber::VersionNumber(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, VersionNumber_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.VersionNumber)
}
VersionNumber::VersionNumber(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const VersionNumber& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, VersionNumber_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE VersionNumber::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void VersionNumber::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, major__),
           0,
           offsetof(Impl_, patch__) -
               offsetof(Impl_, major__) +
               sizeof(Impl_::patch__));
}
VersionNumber::~VersionNumber() {
  // @@protoc_insertion_point(destructor:mgard.pb.VersionNumber)
  SharedDtor(*this);
}
inline void VersionNumber::SharedDtor(MessageLite& self) {
  VersionNumber& this_ = static_cast<VersionNumber&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull VersionNumber_class_data_ =
        VersionNumber::InternalGenerateClassData_(VersionNumber_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
VersionNumber::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&VersionNumber_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(VersionNumber_class_data_.tc_table);
  return VersionNumber_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
VersionNumber::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&VersionNumber_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&VersionNumber_globals_));
  return VersionNumber_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const VersionNumber::ParseTableT_
    VersionNumber::_table_ =
        VersionNumber::InternalGenerateParseTable_(VersionNumber_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void VersionNumber::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.VersionNumber)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    ::memset(&this_._impl_.major__, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.patch__) -
                 reinterpret_cast<char*>(&this_._impl_.major__)) +
                 sizeof(_impl_.patch__));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL VersionNumber::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const VersionNumber& this_ = static_cast<const VersionNumber&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL VersionNumber::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const VersionNumber& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.VersionNumber)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint64 major_ = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_major_() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          1, this_._internal_major_(), target);
    }
  }

  // uint64 minor_ = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_minor_() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_minor_(), target);
    }
  }

  // uint64 patch_ = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_patch_() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_patch_(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.VersionNumber)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t VersionNumber::ByteSizeLong(const MessageLite& base) {
  const VersionNumber& this_ = static_cast<const VersionNumber&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t VersionNumber::ByteSizeLong() const {
  const VersionNumber& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.VersionNumber)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // uint64 major_ = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_major_() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_major_());
      }
    }
    // uint64 minor_ = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_minor_() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_minor_());
      }
    }
    // uint64 patch_ = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_patch_() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_patch_());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void VersionNumber::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<VersionNumber*>(&to_msg);
  auto& from = static_cast<const VersionNumber&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.VersionNumber)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_major_() != 0) {
        _this->_impl_.major__ = from._impl_.major__;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_minor_() != 0) {
        _this->_impl_.minor__ = from._impl_.minor__;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_patch_() != 0) {
        _this->_impl_.patch__ = from._impl_.patch__;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void VersionNumber::CopyFrom(const VersionNumber& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.VersionNumber)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void VersionNumber::InternalSwap(VersionNumber* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.patch__)
      + sizeof(VersionNumber::_impl_.patch__)
      - PROTOBUF_FIELD_OFFSET(VersionNumber, _impl_.major__)>(
          reinterpret_cast<char*>(&_impl_.major__),
          reinterpret_cast<char*>(&other->_impl_.major__));
}

::google::protobuf::Metadata VersionNumber::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

CartesianGridTopology::CartesianGridTopology(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, CartesianGridTopology_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.CartesianGridTopology)
}
PROTOBUF_NDEBUG_INLINE CartesianGridTopology::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mgard::pb::CartesianGridTopology& from_msg)
      : _has_bits_{from._has_bits_},
        shape_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::mgard::pb::CartesianGridTopology,
              PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_.shape_)>()
          , arena, from.shape_
        }
     {}

CartesianGridTopology::CartesianGridTopology(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CartesianGridTopology& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, CartesianGridTopology_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CartesianGridTopology* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  _impl_.dimension_ = from._impl_.dimension_;

  // @@protoc_insertion_point(copy_constructor:mgard.pb.CartesianGridTopology)
}
PROTOBUF_NDEBUG_INLINE CartesianGridTopology::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : shape_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mgard::pb::CartesianGridTopology,
            PROTOBUF_FIELD_OFFSET(::mgard::pb::CartesianGridTopology, _impl_.shape_)>()
         }
     {}

inline void CartesianGridTopology::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.dimension_ = {};
}
CartesianGridTopology::~CartesianGridTopology() {
  // @@protoc_insertion_point(destructor:mgard.pb.CartesianGridTopology)
  SharedDtor(*this);
}
inline void CartesianGridTopology::SharedDtor(MessageLite& self) {
  CartesianGridTopology& this_ = static_cast<CartesianGridTopology&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CartesianGridTopology_class_data_ =
        CartesianGridTopology::InternalGenerateClassData_(CartesianGridTopology_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CartesianGridTopology::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CartesianGridTopology_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CartesianGridTopology_class_data_.tc_table);
  return CartesianGridTopology_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CartesianGridTopology::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CartesianGridTopology_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CartesianGridTopology_globals_));
  return CartesianGridTopology_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CartesianGridTopology::ParseTableT_
    CartesianGridTopology::_table_ =
        CartesianGridTopology::InternalGenerateParseTable_(CartesianGridTopology_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CartesianGridTopology::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.CartesianGridTopology)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.shape_.Clear();
  }
  this_._impl_.dimension_ = ::uint64_t{0u};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CartesianGridTopology::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CartesianGridTopology& this_ = static_cast<const CartesianGridTopology&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CartesianGridTopology::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CartesianGridTopology& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.CartesianGridTopology)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint64 dimension = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_dimension() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          1, this_._internal_dimension(), target);
    }
  }

  // repeated uint64 shape = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    {
      int byte_size = this_._impl_._shape_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteUInt64Packed(
            2, this_._internal_shape(), byte_size, target);
      }
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.CartesianGridTopology)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CartesianGridTopology::ByteSizeLong(const MessageLite& base) {
  const CartesianGridTopology& this_ = static_cast<const CartesianGridTopology&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CartesianGridTopology::ByteSizeLong() const {
  const CartesianGridTopology& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.CartesianGridTopology)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // repeated uint64 shape = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          ::_pbi::WireFormatLite::UInt64SizeWithPackedTagSize(
              this_._internal_shape(), 1,
              this_._impl_._shape_cached_byte_size_);
    }
    // uint64 dimension = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_dimension() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_dimension());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CartesianGridTopology::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CartesianGridTopology*>(&to_msg);
  auto& from = static_cast<const CartesianGridTopology&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.CartesianGridTopology)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_shape()->MergeFrom(from._internal_shape());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_dimension() != 0) {
        _this->_impl_.dimension_ = from._impl_.dimension_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CartesianGridTopology::CopyFrom(const CartesianGridTopology& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.CartesianGridTopology)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CartesianGridTopology::InternalSwap(CartesianGridTopology* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.shape_.InternalSwap(&other->_impl_.shape_);
  swap(_impl_.dimension_, other->_impl_.dimension_);
}

::google::protobuf::Metadata CartesianGridTopology::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ExplicitCubeGeometry::ExplicitCubeGeometry(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExplicitCubeGeometry_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.ExplicitCubeGeometry)
}
PROTOBUF_NDEBUG_INLINE ExplicitCubeGeometry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mgard::pb::ExplicitCubeGeometry& from_msg)
      : _has_bits_{from._has_bits_},
        coordinates_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::mgard::pb::ExplicitCubeGeometry,
              PROTOBUF_FIELD_OFFSET(::mgard::pb::ExplicitCubeGeometry, _impl_.coordinates_)>()
          , arena, from.coordinates_
        }
     {}

ExplicitCubeGeometry::ExplicitCubeGeometry(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExplicitCubeGeometry& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExplicitCubeGeometry_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExplicitCubeGeometry* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:mgard.pb.ExplicitCubeGeometry)
}
PROTOBUF_NDEBUG_INLINE ExplicitCubeGeometry::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : coordinates_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::mgard::pb::ExplicitCubeGeometry,
            PROTOBUF_FIELD_OFFSET(::mgard::pb::ExplicitCubeGeometry, _impl_.coordinates_)>()
         }
     {}

inline void ExplicitCubeGeometry::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
ExplicitCubeGeometry::~ExplicitCubeGeometry() {
  // @@protoc_insertion_point(destructor:mgard.pb.ExplicitCubeGeometry)
  SharedDtor(*this);
}
inline void ExplicitCubeGeometry::SharedDtor(MessageLite& self) {
  ExplicitCubeGeometry& this_ = static_cast<ExplicitCubeGeometry&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExplicitCubeGeometry_class_data_ =
        ExplicitCubeGeometry::InternalGenerateClassData_(ExplicitCubeGeometry_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExplicitCubeGeometry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExplicitCubeGeometry_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExplicitCubeGeometry_class_data_.tc_table);
  return ExplicitCubeGeometry_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExplicitCubeGeometry::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExplicitCubeGeometry_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExplicitCubeGeometry_globals_));
  return ExplicitCubeGeometry_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExplicitCubeGeometry::ParseTableT_
    ExplicitCubeGeometry::_table_ =
        ExplicitCubeGeometry::InternalGenerateParseTable_(ExplicitCubeGeometry_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExplicitCubeGeometry::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.ExplicitCubeGeometry)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _impl_.coordinates_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExplicitCubeGeometry::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExplicitCubeGeometry& this_ = static_cast<const ExplicitCubeGeometry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExplicitCubeGeometry::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExplicitCubeGeometry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.ExplicitCubeGeometry)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated double coordinates = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_coordinates_size() > 0) {
      target = stream->WriteFixedPacked(2, this_._internal_coordinates(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.ExplicitCubeGeometry)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExplicitCubeGeometry::ByteSizeLong(const MessageLite& base) {
  const ExplicitCubeGeometry& this_ = static_cast<const ExplicitCubeGeometry&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExplicitCubeGeometry::ByteSizeLong() const {
  const ExplicitCubeGeometry& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.ExplicitCubeGeometry)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated double coordinates = 2;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ::size_t data_size = ::size_t{8} *
          ::_pbi::FromIntSize(this_._internal_coordinates_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExplicitCubeGeometry::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExplicitCubeGeometry*>(&to_msg);
  auto& from = static_cast<const ExplicitCubeGeometry&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.ExplicitCubeGeometry)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_coordinates()->MergeFrom(from._internal_coordinates());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExplicitCubeGeometry::CopyFrom(const ExplicitCubeGeometry& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.ExplicitCubeGeometry)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExplicitCubeGeometry::InternalSwap(ExplicitCubeGeometry* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.coordinates_.InternalSwap(&other->_impl_.coordinates_);
}

::google::protobuf::Metadata ExplicitCubeGeometry::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Domain::set_allocated_cartesian_grid_topology(::mgard::pb::CartesianGridTopology* PROTOBUF_NULLABLE cartesian_grid_topology) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_topology_definition();
  if (cartesian_grid_topology) {
    ::google::protobuf::Arena* submessage_arena = cartesian_grid_topology->GetArena();
    if (message_arena != submessage_arena) {
      cartesian_grid_topology = ::google::protobuf::internal::GetOwnedMessage(message_arena, cartesian_grid_topology, submessage_arena);
    }
    set_has_cartesian_grid_topology();
    _impl_.topology_definition_.cartesian_grid_topology_ = cartesian_grid_topology;
  }
  // @@protoc_insertion_point(field_set_allocated:mgard.pb.Domain.cartesian_grid_topology)
}
void Domain::set_allocated_explicit_cube_geometry(::mgard::pb::ExplicitCubeGeometry* PROTOBUF_NULLABLE explicit_cube_geometry) {
  ::google::protobuf::Arena* message_arena = GetArena();
  clear_geometry_definition();
  if (explicit_cube_geometry) {
    ::google::protobuf::Arena* submessage_arena = explicit_cube_geometry->GetArena();
    if (message_arena != submessage_arena) {
      explicit_cube_geometry = ::google::protobuf::internal::GetOwnedMessage(message_arena, explicit_cube_geometry, submessage_arena);
    }
    set_has_explicit_cube_geometry();
    _impl_.geometry_definition_.explicit_cube_geometry_ = explicit_cube_geometry;
  }
  // @@protoc_insertion_point(field_set_allocated:mgard.pb.Domain.explicit_cube_geometry)
}
Domain::Domain(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Domain_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Domain)
}
PROTOBUF_NDEBUG_INLINE Domain::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mgard::pb::Domain& from_msg)
      : _has_bits_{from._has_bits_},
        topology_definition_{},
        geometry_definition_{},
        _oneof_case_{from._oneof_case_[0], from._oneof_case_[1]} {}

Domain::Domain(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Domain& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Domain_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Domain* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, topology_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, topology_),
           offsetof(Impl_, geometry_) -
               offsetof(Impl_, topology_) +
               sizeof(Impl_::geometry_));
  switch (topology_definition_case()) {
    case TOPOLOGY_DEFINITION_NOT_SET:
      break;
      case kCartesianGridTopology:
        _impl_.topology_definition_.cartesian_grid_topology_ = Super_::CopyConstruct(arena, *from._impl_.topology_definition_.cartesian_grid_topology_);
        break;
  }
  switch (geometry_definition_case()) {
    case GEOMETRY_DEFINITION_NOT_SET:
      break;
      case kExplicitCubeGeometry:
        _impl_.geometry_definition_.explicit_cube_geometry_ = Super_::CopyConstruct(arena, *from._impl_.geometry_definition_.explicit_cube_geometry_);
        break;
      case kExplicitCubeFilename:
        new (&_impl_.geometry_definition_.explicit_cube_filename_) decltype(_impl_.geometry_definition_.explicit_cube_filename_){arena, from._impl_.geometry_definition_.explicit_cube_filename_};
        break;
  }

  // @@protoc_insertion_point(copy_constructor:mgard.pb.Domain)
}
PROTOBUF_NDEBUG_INLINE Domain::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : topology_definition_{},
        geometry_definition_{},
        _oneof_case_{} {}

inline void Domain::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, topology_),
           0,
           offsetof(Impl_, geometry_) -
               offsetof(Impl_, topology_) +
               sizeof(Impl_::geometry_));
}
Domain::~Domain() {
  // @@protoc_insertion_point(destructor:mgard.pb.Domain)
  SharedDtor(*this);
}
inline void Domain::SharedDtor(MessageLite& self) {
  Domain& this_ = static_cast<Domain&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  if (this_.has_topology_definition()) {
    this_.clear_topology_definition();
  }
  if (this_.has_geometry_definition()) {
    this_.clear_geometry_definition();
  }
  this_._impl_.~Impl_();
}

void Domain::clear_topology_definition() {
// @@protoc_insertion_point(one_of_clear_start:mgard.pb.Domain)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (topology_definition_case()) {
    case kCartesianGridTopology: {
      if (GetArena() == nullptr) {
        delete _impl_.topology_definition_.cartesian_grid_topology_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.topology_definition_.cartesian_grid_topology_);
      }
      break;
    }
    case TOPOLOGY_DEFINITION_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[0] = TOPOLOGY_DEFINITION_NOT_SET;
}

void Domain::clear_geometry_definition() {
// @@protoc_insertion_point(one_of_clear_start:mgard.pb.Domain)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  switch (geometry_definition_case()) {
    case kExplicitCubeGeometry: {
      if (GetArena() == nullptr) {
        delete _impl_.geometry_definition_.explicit_cube_geometry_;
      } else if (::google::protobuf::internal::DebugHardenClearOneofMessageOnArena()) {
        ::google::protobuf::internal::MaybePoisonAfterClear(_impl_.geometry_definition_.explicit_cube_geometry_);
      }
      break;
    }
    case kExplicitCubeFilename: {
      _impl_.geometry_definition_.explicit_cube_filename_.Destroy();
      break;
    }
    case GEOMETRY_DEFINITION_NOT_SET: {
      break;
    }
  }
  _impl_._oneof_case_[1] = GEOMETRY_DEFINITION_NOT_SET;
}


#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Domain_class_data_ =
        Domain::InternalGenerateClassData_(Domain_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Domain::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Domain_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Domain_class_data_.tc_table);
  return Domain_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Domain::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Domain_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Domain_globals_));
  return Domain_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Domain::ParseTableT_
    Domain::_table_ =
        Domain::InternalGenerateParseTable_(Domain_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Domain::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Domain)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&this_._impl_.topology_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.geometry_) -
                 reinterpret_cast<char*>(&this_._impl_.topology_)) +
                 sizeof(_impl_.geometry_));
  }
  clear_topology_definition();
  clear_geometry_definition();
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Domain::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Domain& this_ = static_cast<const Domain&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Domain::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Domain& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Domain)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.Domain.Topology topology = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_topology() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_topology(), target);
    }
  }

  // .mgard.pb.CartesianGridTopology cartesian_grid_topology = 2;
  if (this_.topology_definition_case() == kCartesianGridTopology) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.topology_definition_.cartesian_grid_topology_, this_._impl_.topology_definition_.cartesian_grid_topology_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Domain.Geometry geometry = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_geometry() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          3, this_._internal_geometry(), target);
    }
  }

  switch (this_.geometry_definition_case()) {
    case kExplicitCubeGeometry: {
      target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
          4, *this_._impl_.geometry_definition_.explicit_cube_geometry_, this_._impl_.geometry_definition_.explicit_cube_geometry_->GetCachedSize(), target,
          stream);
      break;
    }
    case kExplicitCubeFilename: {
      const ::std::string& _s = this_._internal_explicit_cube_filename();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "mgard.pb.Domain.explicit_cube_filename");
      target = stream->WriteStringMaybeAliased(5, _s, target);
      break;
    }
    default:
      break;
  }
  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Domain)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Domain::ByteSizeLong(const MessageLite& base) {
  const Domain& this_ = static_cast<const Domain&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Domain::ByteSizeLong() const {
  const Domain& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Domain)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .mgard.pb.Domain.Topology topology = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_topology() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_topology());
      }
    }
    // .mgard.pb.Domain.Geometry geometry = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_geometry() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_geometry());
      }
    }
  }
  switch (this_.topology_definition_case()) {
    // .mgard.pb.CartesianGridTopology cartesian_grid_topology = 2;
    case kCartesianGridTopology: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.topology_definition_.cartesian_grid_topology_);
      break;
    }
    case TOPOLOGY_DEFINITION_NOT_SET: {
      break;
    }
  }
  switch (this_.geometry_definition_case()) {
    // .mgard.pb.ExplicitCubeGeometry explicit_cube_geometry = 4;
    case kExplicitCubeGeometry: {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.geometry_definition_.explicit_cube_geometry_);
      break;
    }
    // string explicit_cube_filename = 5;
    case kExplicitCubeFilename: {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_explicit_cube_filename());
      break;
    }
    case GEOMETRY_DEFINITION_NOT_SET: {
      break;
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Domain::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Domain*>(&to_msg);
  auto& from = static_cast<const Domain&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Domain)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_topology() != 0) {
        _this->_impl_.topology_ = from._impl_.topology_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_geometry() != 0) {
        _this->_impl_.geometry_ = from._impl_.geometry_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[0]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[0];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_topology_definition();
      }
      _this->_impl_._oneof_case_[0] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kCartesianGridTopology: {
        if (oneof_needs_init) {
          _this->_impl_.topology_definition_.cartesian_grid_topology_ = Super_::CopyConstruct(arena, *from._impl_.topology_definition_.cartesian_grid_topology_);
        } else {
          _this->_impl_.topology_definition_.cartesian_grid_topology_->MergeFrom(*from._impl_.topology_definition_.cartesian_grid_topology_);
        }
        break;
      }
      case TOPOLOGY_DEFINITION_NOT_SET:
        break;
    }
  }
  if (const uint32_t oneof_from_case =
          from._impl_._oneof_case_[1]) {
    const uint32_t oneof_to_case = _this->_impl_._oneof_case_[1];
    const bool oneof_needs_init = oneof_to_case != oneof_from_case;
    if (oneof_needs_init) {
      if (oneof_to_case != 0) {
        _this->clear_geometry_definition();
      }
      _this->_impl_._oneof_case_[1] = oneof_from_case;
    }

    switch (oneof_from_case) {
      case kExplicitCubeGeometry: {
        if (oneof_needs_init) {
          _this->_impl_.geometry_definition_.explicit_cube_geometry_ = Super_::CopyConstruct(arena, *from._impl_.geometry_definition_.explicit_cube_geometry_);
        } else {
          _this->_impl_.geometry_definition_.explicit_cube_geometry_->MergeFrom(*from._impl_.geometry_definition_.explicit_cube_geometry_);
        }
        break;
      }
      case kExplicitCubeFilename: {
        if (oneof_needs_init) {
          _this->_impl_.geometry_definition_.explicit_cube_filename_.InitDefault();
        }
        _this->_impl_.geometry_definition_.explicit_cube_filename_.Set(from._internal_explicit_cube_filename(), arena);
        break;
      }
      case GEOMETRY_DEFINITION_NOT_SET:
        break;
    }
  }
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Domain::CopyFrom(const Domain& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Domain)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Domain::InternalSwap(Domain* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Domain, _impl_.geometry_)
      + sizeof(Domain::_impl_.geometry_)
      - PROTOBUF_FIELD_OFFSET(Domain, _impl_.topology_)>(
          reinterpret_cast<char*>(&_impl_.topology_),
          reinterpret_cast<char*>(&other->_impl_.topology_));
  swap(_impl_.topology_definition_, other->_impl_.topology_definition_);
  swap(_impl_.geometry_definition_, other->_impl_.geometry_definition_);
  swap(_impl_._oneof_case_[0], other->_impl_._oneof_case_[0]);
  swap(_impl_._oneof_case_[1], other->_impl_._oneof_case_[1]);
}

::google::protobuf::Metadata Domain::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Dataset::Dataset(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Dataset_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Dataset)
}
Dataset::Dataset(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Dataset& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Dataset_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Dataset::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Dataset::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, dimension_),
           0,
           offsetof(Impl_, type_) -
               offsetof(Impl_, dimension_) +
               sizeof(Impl_::type_));
}
Dataset::~Dataset() {
  // @@protoc_insertion_point(destructor:mgard.pb.Dataset)
  SharedDtor(*this);
}
inline void Dataset::SharedDtor(MessageLite& self) {
  Dataset& this_ = static_cast<Dataset&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Dataset_class_data_ =
        Dataset::InternalGenerateClassData_(Dataset_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Dataset::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Dataset_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Dataset_class_data_.tc_table);
  return Dataset_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Dataset::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Dataset_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Dataset_globals_));
  return Dataset_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Dataset::ParseTableT_
    Dataset::_table_ =
        Dataset::InternalGenerateParseTable_(Dataset_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Dataset::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Dataset)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&this_._impl_.dimension_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.type_) -
                 reinterpret_cast<char*>(&this_._impl_.dimension_)) +
                 sizeof(_impl_.type_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Dataset::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Dataset& this_ = static_cast<const Dataset&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Dataset::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Dataset& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Dataset)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.Dataset.Type type = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_type(), target);
    }
  }

  // uint64 dimension = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_dimension() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_dimension(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Dataset)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Dataset::ByteSizeLong(const MessageLite& base) {
  const Dataset& this_ = static_cast<const Dataset&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Dataset::ByteSizeLong() const {
  const Dataset& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Dataset)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // uint64 dimension = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_dimension() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_dimension());
      }
    }
    // .mgard.pb.Dataset.Type type = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Dataset::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Dataset*>(&to_msg);
  auto& from = static_cast<const Dataset&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Dataset)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_dimension() != 0) {
        _this->_impl_.dimension_ = from._impl_.dimension_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Dataset::CopyFrom(const Dataset& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Dataset)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Dataset::InternalSwap(Dataset* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Dataset, _impl_.type_)
      + sizeof(Dataset::_impl_.type_)
      - PROTOBUF_FIELD_OFFSET(Dataset, _impl_.dimension_)>(
          reinterpret_cast<char*>(&_impl_.dimension_),
          reinterpret_cast<char*>(&other->_impl_.dimension_));
}

::google::protobuf::Metadata Dataset::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ErrorControl::ErrorControl(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ErrorControl_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.ErrorControl)
}
ErrorControl::ErrorControl(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ErrorControl& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ErrorControl_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE ErrorControl::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void ErrorControl::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, mode_),
           0,
           offsetof(Impl_, tolerance_) -
               offsetof(Impl_, mode_) +
               sizeof(Impl_::tolerance_));
}
ErrorControl::~ErrorControl() {
  // @@protoc_insertion_point(destructor:mgard.pb.ErrorControl)
  SharedDtor(*this);
}
inline void ErrorControl::SharedDtor(MessageLite& self) {
  ErrorControl& this_ = static_cast<ErrorControl&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ErrorControl_class_data_ =
        ErrorControl::InternalGenerateClassData_(ErrorControl_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ErrorControl::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ErrorControl_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ErrorControl_class_data_.tc_table);
  return ErrorControl_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ErrorControl::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ErrorControl_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ErrorControl_globals_));
  return ErrorControl_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ErrorControl::ParseTableT_
    ErrorControl::_table_ =
        ErrorControl::InternalGenerateParseTable_(ErrorControl_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ErrorControl::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.ErrorControl)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    ::memset(&this_._impl_.mode_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.tolerance_) -
                 reinterpret_cast<char*>(&this_._impl_.mode_)) +
                 sizeof(_impl_.tolerance_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ErrorControl::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ErrorControl& this_ = static_cast<const ErrorControl&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ErrorControl::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ErrorControl& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.ErrorControl)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.ErrorControl.Mode mode = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_mode() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_mode(), target);
    }
  }

  // .mgard.pb.ErrorControl.Norm norm = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_norm() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_norm(), target);
    }
  }

  // double s = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_s()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          3, this_._internal_s(), target);
    }
  }

  // double norm_of_original_data = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_norm_of_original_data()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          4, this_._internal_norm_of_original_data(), target);
    }
  }

  // double tolerance = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (::absl::bit_cast<::uint64_t>(this_._internal_tolerance()) != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteDoubleToArray(
          5, this_._internal_tolerance(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.ErrorControl)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ErrorControl::ByteSizeLong(const MessageLite& base) {
  const ErrorControl& this_ = static_cast<const ErrorControl&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ErrorControl::ByteSizeLong() const {
  const ErrorControl& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.ErrorControl)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // .mgard.pb.ErrorControl.Mode mode = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_mode() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_mode());
      }
    }
    // .mgard.pb.ErrorControl.Norm norm = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_norm() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_norm());
      }
    }
    // double s = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_s()) != 0) {
        total_size += 9;
      }
    }
    // double norm_of_original_data = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_norm_of_original_data()) != 0) {
        total_size += 9;
      }
    }
    // double tolerance = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (::absl::bit_cast<::uint64_t>(this_._internal_tolerance()) != 0) {
        total_size += 9;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ErrorControl::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ErrorControl*>(&to_msg);
  auto& from = static_cast<const ErrorControl&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.ErrorControl)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_mode() != 0) {
        _this->_impl_.mode_ = from._impl_.mode_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_norm() != 0) {
        _this->_impl_.norm_ = from._impl_.norm_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_s()) != 0) {
        _this->_impl_.s_ = from._impl_.s_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_norm_of_original_data()) != 0) {
        _this->_impl_.norm_of_original_data_ = from._impl_.norm_of_original_data_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (::absl::bit_cast<::uint64_t>(from._internal_tolerance()) != 0) {
        _this->_impl_.tolerance_ = from._impl_.tolerance_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ErrorControl::CopyFrom(const ErrorControl& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.ErrorControl)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ErrorControl::InternalSwap(ErrorControl* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.tolerance_)
      + sizeof(ErrorControl::_impl_.tolerance_)
      - PROTOBUF_FIELD_OFFSET(ErrorControl, _impl_.mode_)>(
          reinterpret_cast<char*>(&_impl_.mode_),
          reinterpret_cast<char*>(&other->_impl_.mode_));
}

::google::protobuf::Metadata ErrorControl::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

DomainDecomposition::DomainDecomposition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, DomainDecomposition_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.DomainDecomposition)
}
DomainDecomposition::DomainDecomposition(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const DomainDecomposition& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, DomainDecomposition_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE DomainDecomposition::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void DomainDecomposition::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, decomposition_dimension_),
           0,
           offsetof(Impl_, method_) -
               offsetof(Impl_, decomposition_dimension_) +
               sizeof(Impl_::method_));
}
DomainDecomposition::~DomainDecomposition() {
  // @@protoc_insertion_point(destructor:mgard.pb.DomainDecomposition)
  SharedDtor(*this);
}
inline void DomainDecomposition::SharedDtor(MessageLite& self) {
  DomainDecomposition& this_ = static_cast<DomainDecomposition&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull DomainDecomposition_class_data_ =
        DomainDecomposition::InternalGenerateClassData_(DomainDecomposition_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DomainDecomposition::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DomainDecomposition_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(DomainDecomposition_class_data_.tc_table);
  return DomainDecomposition_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
DomainDecomposition::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&DomainDecomposition_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&DomainDecomposition_globals_));
  return DomainDecomposition_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const DomainDecomposition::ParseTableT_
    DomainDecomposition::_table_ =
        DomainDecomposition::InternalGenerateParseTable_(DomainDecomposition_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void DomainDecomposition::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.DomainDecomposition)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    ::memset(&this_._impl_.decomposition_dimension_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.method_) -
                 reinterpret_cast<char*>(&this_._impl_.decomposition_dimension_)) +
                 sizeof(_impl_.method_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL DomainDecomposition::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const DomainDecomposition& this_ = static_cast<const DomainDecomposition&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL DomainDecomposition::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const DomainDecomposition& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.DomainDecomposition)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.DomainDecomposition.Method method = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_method() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_method(), target);
    }
  }

  // uint64 decomposition_dimension = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_decomposition_dimension() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_decomposition_dimension(), target);
    }
  }

  // uint64 decomposition_size = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_decomposition_size() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_decomposition_size(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.DomainDecomposition)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t DomainDecomposition::ByteSizeLong(const MessageLite& base) {
  const DomainDecomposition& this_ = static_cast<const DomainDecomposition&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t DomainDecomposition::ByteSizeLong() const {
  const DomainDecomposition& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.DomainDecomposition)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // uint64 decomposition_dimension = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_decomposition_dimension() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_decomposition_dimension());
      }
    }
    // uint64 decomposition_size = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_decomposition_size() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_decomposition_size());
      }
    }
    // .mgard.pb.DomainDecomposition.Method method = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_method() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_method());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void DomainDecomposition::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<DomainDecomposition*>(&to_msg);
  auto& from = static_cast<const DomainDecomposition&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.DomainDecomposition)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_decomposition_dimension() != 0) {
        _this->_impl_.decomposition_dimension_ = from._impl_.decomposition_dimension_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_decomposition_size() != 0) {
        _this->_impl_.decomposition_size_ = from._impl_.decomposition_size_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_method() != 0) {
        _this->_impl_.method_ = from._impl_.method_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void DomainDecomposition::CopyFrom(const DomainDecomposition& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.DomainDecomposition)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void DomainDecomposition::InternalSwap(DomainDecomposition* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.method_)
      + sizeof(DomainDecomposition::_impl_.method_)
      - PROTOBUF_FIELD_OFFSET(DomainDecomposition, _impl_.decomposition_dimension_)>(
          reinterpret_cast<char*>(&_impl_.decomposition_dimension_),
          reinterpret_cast<char*>(&other->_impl_.decomposition_dimension_));
}

::google::protobuf::Metadata DomainDecomposition::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

FunctionDecomposition::FunctionDecomposition(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FunctionDecomposition_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.FunctionDecomposition)
}
FunctionDecomposition::FunctionDecomposition(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const FunctionDecomposition& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, FunctionDecomposition_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE FunctionDecomposition::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void FunctionDecomposition::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, transform_),
           0,
           offsetof(Impl_, l_target_) -
               offsetof(Impl_, transform_) +
               sizeof(Impl_::l_target_));
}
FunctionDecomposition::~FunctionDecomposition() {
  // @@protoc_insertion_point(destructor:mgard.pb.FunctionDecomposition)
  SharedDtor(*this);
}
inline void FunctionDecomposition::SharedDtor(MessageLite& self) {
  FunctionDecomposition& this_ = static_cast<FunctionDecomposition&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull FunctionDecomposition_class_data_ =
        FunctionDecomposition::InternalGenerateClassData_(FunctionDecomposition_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FunctionDecomposition::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FunctionDecomposition_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(FunctionDecomposition_class_data_.tc_table);
  return FunctionDecomposition_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
FunctionDecomposition::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&FunctionDecomposition_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&FunctionDecomposition_globals_));
  return FunctionDecomposition_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const FunctionDecomposition::ParseTableT_
    FunctionDecomposition::_table_ =
        FunctionDecomposition::InternalGenerateParseTable_(FunctionDecomposition_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void FunctionDecomposition::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.FunctionDecomposition)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    ::memset(&this_._impl_.transform_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.l_target_) -
                 reinterpret_cast<char*>(&this_._impl_.transform_)) +
                 sizeof(_impl_.l_target_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL FunctionDecomposition::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const FunctionDecomposition& this_ = static_cast<const FunctionDecomposition&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL FunctionDecomposition::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const FunctionDecomposition& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.FunctionDecomposition)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.FunctionDecomposition.Transform transform = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_transform() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_transform(), target);
    }
  }

  // .mgard.pb.FunctionDecomposition.Hierarchy hierarchy = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_hierarchy() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_hierarchy(), target);
    }
  }

  // uint64 L_target = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_l_target() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_l_target(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.FunctionDecomposition)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t FunctionDecomposition::ByteSizeLong(const MessageLite& base) {
  const FunctionDecomposition& this_ = static_cast<const FunctionDecomposition&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t FunctionDecomposition::ByteSizeLong() const {
  const FunctionDecomposition& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.FunctionDecomposition)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // .mgard.pb.FunctionDecomposition.Transform transform = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_transform() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_transform());
      }
    }
    // .mgard.pb.FunctionDecomposition.Hierarchy hierarchy = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_hierarchy() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_hierarchy());
      }
    }
    // uint64 L_target = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_l_target() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_l_target());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void FunctionDecomposition::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<FunctionDecomposition*>(&to_msg);
  auto& from = static_cast<const FunctionDecomposition&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.FunctionDecomposition)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_transform() != 0) {
        _this->_impl_.transform_ = from._impl_.transform_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_hierarchy() != 0) {
        _this->_impl_.hierarchy_ = from._impl_.hierarchy_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_l_target() != 0) {
        _this->_impl_.l_target_ = from._impl_.l_target_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void FunctionDecomposition::CopyFrom(const FunctionDecomposition& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.FunctionDecomposition)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void FunctionDecomposition::InternalSwap(FunctionDecomposition* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.l_target_)
      + sizeof(FunctionDecomposition::_impl_.l_target_)
      - PROTOBUF_FIELD_OFFSET(FunctionDecomposition, _impl_.transform_)>(
          reinterpret_cast<char*>(&_impl_.transform_),
          reinterpret_cast<char*>(&other->_impl_.transform_));
}

::google::protobuf::Metadata FunctionDecomposition::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Quantization::Quantization(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Quantization_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Quantization)
}
Quantization::Quantization(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Quantization& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Quantization_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Quantization::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Quantization::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, method_),
           0,
           offsetof(Impl_, big_endian_) -
               offsetof(Impl_, method_) +
               sizeof(Impl_::big_endian_));
}
Quantization::~Quantization() {
  // @@protoc_insertion_point(destructor:mgard.pb.Quantization)
  SharedDtor(*this);
}
inline void Quantization::SharedDtor(MessageLite& self) {
  Quantization& this_ = static_cast<Quantization&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Quantization_class_data_ =
        Quantization::InternalGenerateClassData_(Quantization_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Quantization::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Quantization_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Quantization_class_data_.tc_table);
  return Quantization_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Quantization::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Quantization_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Quantization_globals_));
  return Quantization_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Quantization::ParseTableT_
    Quantization::_table_ =
        Quantization::InternalGenerateParseTable_(Quantization_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Quantization::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Quantization)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    ::memset(&this_._impl_.method_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.big_endian_) -
                 reinterpret_cast<char*>(&this_._impl_.method_)) +
                 sizeof(_impl_.big_endian_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Quantization::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Quantization& this_ = static_cast<const Quantization&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Quantization::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Quantization& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Quantization)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.Quantization.Method method = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_method() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_method(), target);
    }
  }

  // .mgard.pb.Quantization.BinWidths bin_widths = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_bin_widths() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_bin_widths(), target);
    }
  }

  // .mgard.pb.Quantization.Type type = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          3, this_._internal_type(), target);
    }
  }

  // bool big_endian = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_big_endian() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          4, this_._internal_big_endian(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Quantization)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Quantization::ByteSizeLong(const MessageLite& base) {
  const Quantization& this_ = static_cast<const Quantization&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Quantization::ByteSizeLong() const {
  const Quantization& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Quantization)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .mgard.pb.Quantization.Method method = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_method() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_method());
      }
    }
    // .mgard.pb.Quantization.BinWidths bin_widths = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_bin_widths() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_bin_widths());
      }
    }
    // .mgard.pb.Quantization.Type type = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
    // bool big_endian = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_big_endian() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Quantization::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Quantization*>(&to_msg);
  auto& from = static_cast<const Quantization&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Quantization)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_method() != 0) {
        _this->_impl_.method_ = from._impl_.method_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_bin_widths() != 0) {
        _this->_impl_.bin_widths_ = from._impl_.bin_widths_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_big_endian() != 0) {
        _this->_impl_.big_endian_ = from._impl_.big_endian_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Quantization::CopyFrom(const Quantization& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Quantization)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Quantization::InternalSwap(Quantization* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Quantization, _impl_.big_endian_)
      + sizeof(Quantization::_impl_.big_endian_)
      - PROTOBUF_FIELD_OFFSET(Quantization, _impl_.method_)>(
          reinterpret_cast<char*>(&_impl_.method_),
          reinterpret_cast<char*>(&other->_impl_.method_));
}

::google::protobuf::Metadata Quantization::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

BitplaneEncoding::BitplaneEncoding(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, BitplaneEncoding_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.BitplaneEncoding)
}
BitplaneEncoding::BitplaneEncoding(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const BitplaneEncoding& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, BitplaneEncoding_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE BitplaneEncoding::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void BitplaneEncoding::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, method_),
           0,
           offsetof(Impl_, big_endian_) -
               offsetof(Impl_, method_) +
               sizeof(Impl_::big_endian_));
}
BitplaneEncoding::~BitplaneEncoding() {
  // @@protoc_insertion_point(destructor:mgard.pb.BitplaneEncoding)
  SharedDtor(*this);
}
inline void BitplaneEncoding::SharedDtor(MessageLite& self) {
  BitplaneEncoding& this_ = static_cast<BitplaneEncoding&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull BitplaneEncoding_class_data_ =
        BitplaneEncoding::InternalGenerateClassData_(BitplaneEncoding_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BitplaneEncoding::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BitplaneEncoding_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(BitplaneEncoding_class_data_.tc_table);
  return BitplaneEncoding_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
BitplaneEncoding::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&BitplaneEncoding_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&BitplaneEncoding_globals_));
  return BitplaneEncoding_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const BitplaneEncoding::ParseTableT_
    BitplaneEncoding::_table_ =
        BitplaneEncoding::InternalGenerateParseTable_(BitplaneEncoding_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void BitplaneEncoding::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.BitplaneEncoding)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    ::memset(&this_._impl_.method_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.big_endian_) -
                 reinterpret_cast<char*>(&this_._impl_.method_)) +
                 sizeof(_impl_.big_endian_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL BitplaneEncoding::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const BitplaneEncoding& this_ = static_cast<const BitplaneEncoding&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL BitplaneEncoding::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const BitplaneEncoding& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.BitplaneEncoding)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.BitplaneEncoding.Method method = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_method() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_method(), target);
    }
  }

  // .mgard.pb.BitplaneEncoding.Type type = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_type() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_type(), target);
    }
  }

  // uint64 number_bitplanes = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_number_bitplanes() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_number_bitplanes(), target);
    }
  }

  // bool big_endian = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_big_endian() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          4, this_._internal_big_endian(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.BitplaneEncoding)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t BitplaneEncoding::ByteSizeLong(const MessageLite& base) {
  const BitplaneEncoding& this_ = static_cast<const BitplaneEncoding&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t BitplaneEncoding::ByteSizeLong() const {
  const BitplaneEncoding& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.BitplaneEncoding)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .mgard.pb.BitplaneEncoding.Method method = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_method() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_method());
      }
    }
    // .mgard.pb.BitplaneEncoding.Type type = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_type() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_type());
      }
    }
    // uint64 number_bitplanes = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_number_bitplanes() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_number_bitplanes());
      }
    }
    // bool big_endian = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_big_endian() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void BitplaneEncoding::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<BitplaneEncoding*>(&to_msg);
  auto& from = static_cast<const BitplaneEncoding&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.BitplaneEncoding)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_method() != 0) {
        _this->_impl_.method_ = from._impl_.method_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_number_bitplanes() != 0) {
        _this->_impl_.number_bitplanes_ = from._impl_.number_bitplanes_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_big_endian() != 0) {
        _this->_impl_.big_endian_ = from._impl_.big_endian_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void BitplaneEncoding::CopyFrom(const BitplaneEncoding& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.BitplaneEncoding)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void BitplaneEncoding::InternalSwap(BitplaneEncoding* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.big_endian_)
      + sizeof(BitplaneEncoding::_impl_.big_endian_)
      - PROTOBUF_FIELD_OFFSET(BitplaneEncoding, _impl_.method_)>(
          reinterpret_cast<char*>(&_impl_.method_),
          reinterpret_cast<char*>(&other->_impl_.method_));
}

::google::protobuf::Metadata BitplaneEncoding::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Encoding::Encoding(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Encoding_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Encoding)
}
Encoding::Encoding(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Encoding& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Encoding_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Encoding::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Encoding::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, preprocessor_),
           0,
           offsetof(Impl_, huffman_block_size_) -
               offsetof(Impl_, preprocessor_) +
               sizeof(Impl_::huffman_block_size_));
}
Encoding::~Encoding() {
  // @@protoc_insertion_point(destructor:mgard.pb.Encoding)
  SharedDtor(*this);
}
inline void Encoding::SharedDtor(MessageLite& self) {
  Encoding& this_ = static_cast<Encoding&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Encoding_class_data_ =
        Encoding::InternalGenerateClassData_(Encoding_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Encoding::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Encoding_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Encoding_class_data_.tc_table);
  return Encoding_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Encoding::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Encoding_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Encoding_globals_));
  return Encoding_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Encoding::ParseTableT_
    Encoding::_table_ =
        Encoding::InternalGenerateParseTable_(Encoding_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Encoding::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Encoding)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    ::memset(&this_._impl_.preprocessor_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.huffman_block_size_) -
                 reinterpret_cast<char*>(&this_._impl_.preprocessor_)) +
                 sizeof(_impl_.huffman_block_size_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Encoding::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Encoding& this_ = static_cast<const Encoding&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Encoding::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Encoding& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Encoding)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.Encoding.Preprocessor preprocessor = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_preprocessor() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_preprocessor(), target);
    }
  }

  // .mgard.pb.Encoding.Compressor compressor = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_compressor() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          2, this_._internal_compressor(), target);
    }
  }

  // uint64 huffman_dictionary_size = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_huffman_dictionary_size() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_huffman_dictionary_size(), target);
    }
  }

  // uint64 huffman_block_size = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_huffman_block_size() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          4, this_._internal_huffman_block_size(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Encoding)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Encoding::ByteSizeLong(const MessageLite& base) {
  const Encoding& this_ = static_cast<const Encoding&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Encoding::ByteSizeLong() const {
  const Encoding& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Encoding)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .mgard.pb.Encoding.Preprocessor preprocessor = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_preprocessor() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_preprocessor());
      }
    }
    // .mgard.pb.Encoding.Compressor compressor = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_compressor() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_compressor());
      }
    }
    // uint64 huffman_dictionary_size = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_huffman_dictionary_size() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_huffman_dictionary_size());
      }
    }
    // uint64 huffman_block_size = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_huffman_block_size() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_huffman_block_size());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Encoding::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Encoding*>(&to_msg);
  auto& from = static_cast<const Encoding&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Encoding)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_preprocessor() != 0) {
        _this->_impl_.preprocessor_ = from._impl_.preprocessor_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_compressor() != 0) {
        _this->_impl_.compressor_ = from._impl_.compressor_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_huffman_dictionary_size() != 0) {
        _this->_impl_.huffman_dictionary_size_ = from._impl_.huffman_dictionary_size_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_huffman_block_size() != 0) {
        _this->_impl_.huffman_block_size_ = from._impl_.huffman_block_size_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Encoding::CopyFrom(const Encoding& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Encoding)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Encoding::InternalSwap(Encoding* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Encoding, _impl_.huffman_block_size_)
      + sizeof(Encoding::_impl_.huffman_block_size_)
      - PROTOBUF_FIELD_OFFSET(Encoding, _impl_.preprocessor_)>(
          reinterpret_cast<char*>(&_impl_.preprocessor_),
          reinterpret_cast<char*>(&other->_impl_.preprocessor_));
}

::google::protobuf::Metadata Encoding::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Device::Device(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Device_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Device)
}
Device::Device(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Device& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Device_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Device::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Device::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.backend_ = {};
}
Device::~Device() {
  // @@protoc_insertion_point(destructor:mgard.pb.Device)
  SharedDtor(*this);
}
inline void Device::SharedDtor(MessageLite& self) {
  Device& this_ = static_cast<Device&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Device_class_data_ =
        Device::InternalGenerateClassData_(Device_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Device::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Device_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Device_class_data_.tc_table);
  return Device_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Device::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Device_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Device_globals_));
  return Device_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Device::ParseTableT_
    Device::_table_ =
        Device::InternalGenerateParseTable_(Device_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Device::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Device)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  this_._impl_.backend_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Device::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Device& this_ = static_cast<const Device&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Device::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Device& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Device)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.Device.Backend backend = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_backend() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteEnumToArray(
          1, this_._internal_backend(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Device)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Device::ByteSizeLong(const MessageLite& base) {
  const Device& this_ = static_cast<const Device&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Device::ByteSizeLong() const {
  const Device& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Device)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

   {
    // .mgard.pb.Device.Backend backend = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_backend() != 0) {
        total_size += 1 +
                      ::_pbi::WireFormatLite::EnumSize(this_._internal_backend());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Device::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Device*>(&to_msg);
  auto& from = static_cast<const Device&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Device)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_backend() != 0) {
      _this->_impl_.backend_ = from._impl_.backend_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Device::CopyFrom(const Device& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Device)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Device::InternalSwap(Device* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.backend_, other->_impl_.backend_);
}

::google::protobuf::Metadata Device::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Header::Header(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Header_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:mgard.pb.Header)
}
PROTOBUF_NDEBUG_INLINE Header::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::mgard::pb::Header& from_msg)
      : _has_bits_{from._has_bits_} {}

Header::Header(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Header& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Header_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Header* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.mgard_version_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.mgard_version_)
                 : nullptr;
  _impl_.file_format_version_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.file_format_version_)
                 : nullptr;
  _impl_.domain_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.domain_)
                 : nullptr;
  _impl_.dataset_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.dataset_)
                 : nullptr;
  _impl_.error_control_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                 ? Super_::CopyConstruct(arena, *from._impl_.error_control_)
                 : nullptr;
  _impl_.domain_decomposition_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                 ? Super_::CopyConstruct(arena, *from._impl_.domain_decomposition_)
                 : nullptr;
  _impl_.function_decomposition_ = (CheckHasBit(cached_has_bits, 0x00000040U))
                 ? Super_::CopyConstruct(arena, *from._impl_.function_decomposition_)
                 : nullptr;
  _impl_.quantization_ = (CheckHasBit(cached_has_bits, 0x00000080U))
                 ? Super_::CopyConstruct(arena, *from._impl_.quantization_)
                 : nullptr;
  _impl_.bitplane_encoding_ = (CheckHasBit(cached_has_bits, 0x00000100U))
                 ? Super_::CopyConstruct(arena, *from._impl_.bitplane_encoding_)
                 : nullptr;
  _impl_.encoding_ = (CheckHasBit(cached_has_bits, 0x00000200U))
                 ? Super_::CopyConstruct(arena, *from._impl_.encoding_)
                 : nullptr;
  _impl_.device_ = (CheckHasBit(cached_has_bits, 0x00000400U))
                 ? Super_::CopyConstruct(arena, *from._impl_.device_)
                 : nullptr;

  // @@protoc_insertion_point(copy_constructor:mgard.pb.Header)
}
PROTOBUF_NDEBUG_INLINE Header::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Header::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, mgard_version_),
           0,
           offsetof(Impl_, device_) -
               offsetof(Impl_, mgard_version_) +
               sizeof(Impl_::device_));
}
Header::~Header() {
  // @@protoc_insertion_point(destructor:mgard.pb.Header)
  SharedDtor(*this);
}
inline void Header::SharedDtor(MessageLite& self) {
  Header& this_ = static_cast<Header&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.mgard_version_;
  delete this_._impl_.file_format_version_;
  delete this_._impl_.domain_;
  delete this_._impl_.dataset_;
  delete this_._impl_.error_control_;
  delete this_._impl_.domain_decomposition_;
  delete this_._impl_.function_decomposition_;
  delete this_._impl_.quantization_;
  delete this_._impl_.bitplane_encoding_;
  delete this_._impl_.encoding_;
  delete this_._impl_.device_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Header_class_data_ =
        Header::InternalGenerateClassData_(Header_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Header::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Header_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Header_class_data_.tc_table);
  return Header_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Header::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Header_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Header_globals_));
  return Header_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Header::ParseTableT_
    Header::_table_ =
        Header::InternalGenerateParseTable_(Header_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Header::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:mgard.pb.Header)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.mgard_version_ != nullptr);
      this_._impl_.mgard_version_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.file_format_version_ != nullptr);
      this_._impl_.file_format_version_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.domain_ != nullptr);
      this_._impl_.domain_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.dataset_ != nullptr);
      this_._impl_.dataset_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(this_._impl_.error_control_ != nullptr);
      this_._impl_.error_control_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(this_._impl_.domain_decomposition_ != nullptr);
      this_._impl_.domain_decomposition_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(this_._impl_.function_decomposition_ != nullptr);
      this_._impl_.function_decomposition_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(this_._impl_.quantization_ != nullptr);
      this_._impl_.quantization_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(this_._impl_.bitplane_encoding_ != nullptr);
      this_._impl_.bitplane_encoding_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(this_._impl_.encoding_ != nullptr);
      this_._impl_.encoding_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(this_._impl_.device_ != nullptr);
      this_._impl_.device_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Header::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Header& this_ = static_cast<const Header&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Header::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Header& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:mgard.pb.Header)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .mgard.pb.VersionNumber mgard_version = 2;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.mgard_version_, this_._impl_.mgard_version_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.VersionNumber file_format_version = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        3, *this_._impl_.file_format_version_, this_._impl_.file_format_version_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Domain domain = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        4, *this_._impl_.domain_, this_._impl_.domain_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Dataset dataset = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        5, *this_._impl_.dataset_, this_._impl_.dataset_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.ErrorControl error_control = 6;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.error_control_, this_._impl_.error_control_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.DomainDecomposition domain_decomposition = 7;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        7, *this_._impl_.domain_decomposition_, this_._impl_.domain_decomposition_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.FunctionDecomposition function_decomposition = 8;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        8, *this_._impl_.function_decomposition_, this_._impl_.function_decomposition_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Quantization quantization = 9;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        9, *this_._impl_.quantization_, this_._impl_.quantization_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.BitplaneEncoding bitplane_encoding = 10;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        10, *this_._impl_.bitplane_encoding_, this_._impl_.bitplane_encoding_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Encoding encoding = 11;
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        11, *this_._impl_.encoding_, this_._impl_.encoding_->GetCachedSize(), target,
        stream);
  }

  // .mgard.pb.Device device = 12;
  if (CheckHasBit(cached_has_bits, 0x00000400U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        12, *this_._impl_.device_, this_._impl_.device_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:mgard.pb.Header)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Header::ByteSizeLong(const MessageLite& base) {
  const Header& this_ = static_cast<const Header&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Header::ByteSizeLong() const {
  const Header& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:mgard.pb.Header)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // .mgard.pb.VersionNumber mgard_version = 2;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.mgard_version_);
    }
    // .mgard.pb.VersionNumber file_format_version = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.file_format_version_);
    }
    // .mgard.pb.Domain domain = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.domain_);
    }
    // .mgard.pb.Dataset dataset = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.dataset_);
    }
    // .mgard.pb.ErrorControl error_control = 6;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.error_control_);
    }
    // .mgard.pb.DomainDecomposition domain_decomposition = 7;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.domain_decomposition_);
    }
    // .mgard.pb.FunctionDecomposition function_decomposition = 8;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.function_decomposition_);
    }
    // .mgard.pb.Quantization quantization = 9;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.quantization_);
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    // .mgard.pb.BitplaneEncoding bitplane_encoding = 10;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.bitplane_encoding_);
    }
    // .mgard.pb.Encoding encoding = 11;
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.encoding_);
    }
    // .mgard.pb.Device device = 12;
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.device_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Header::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Header*>(&to_msg);
  auto& from = static_cast<const Header&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:mgard.pb.Header)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.mgard_version_ != nullptr);
      if (_this->_impl_.mgard_version_ == nullptr) {
        _this->_impl_.mgard_version_ = Super_::CopyConstruct(arena, *from._impl_.mgard_version_);
      } else {
        _this->_impl_.mgard_version_->MergeFrom(*from._impl_.mgard_version_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.file_format_version_ != nullptr);
      if (_this->_impl_.file_format_version_ == nullptr) {
        _this->_impl_.file_format_version_ = Super_::CopyConstruct(arena, *from._impl_.file_format_version_);
      } else {
        _this->_impl_.file_format_version_->MergeFrom(*from._impl_.file_format_version_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.domain_ != nullptr);
      if (_this->_impl_.domain_ == nullptr) {
        _this->_impl_.domain_ = Super_::CopyConstruct(arena, *from._impl_.domain_);
      } else {
        _this->_impl_.domain_->MergeFrom(*from._impl_.domain_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.dataset_ != nullptr);
      if (_this->_impl_.dataset_ == nullptr) {
        _this->_impl_.dataset_ = Super_::CopyConstruct(arena, *from._impl_.dataset_);
      } else {
        _this->_impl_.dataset_->MergeFrom(*from._impl_.dataset_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.error_control_ != nullptr);
      if (_this->_impl_.error_control_ == nullptr) {
        _this->_impl_.error_control_ = Super_::CopyConstruct(arena, *from._impl_.error_control_);
      } else {
        _this->_impl_.error_control_->MergeFrom(*from._impl_.error_control_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.domain_decomposition_ != nullptr);
      if (_this->_impl_.domain_decomposition_ == nullptr) {
        _this->_impl_.domain_decomposition_ = Super_::CopyConstruct(arena, *from._impl_.domain_decomposition_);
      } else {
        _this->_impl_.domain_decomposition_->MergeFrom(*from._impl_.domain_decomposition_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      ABSL_DCHECK(from._impl_.function_decomposition_ != nullptr);
      if (_this->_impl_.function_decomposition_ == nullptr) {
        _this->_impl_.function_decomposition_ = Super_::CopyConstruct(arena, *from._impl_.function_decomposition_);
      } else {
        _this->_impl_.function_decomposition_->MergeFrom(*from._impl_.function_decomposition_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      ABSL_DCHECK(from._impl_.quantization_ != nullptr);
      if (_this->_impl_.quantization_ == nullptr) {
        _this->_impl_.quantization_ = Super_::CopyConstruct(arena, *from._impl_.quantization_);
      } else {
        _this->_impl_.quantization_->MergeFrom(*from._impl_.quantization_);
      }
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000700U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(from._impl_.bitplane_encoding_ != nullptr);
      if (_this->_impl_.bitplane_encoding_ == nullptr) {
        _this->_impl_.bitplane_encoding_ = Super_::CopyConstruct(arena, *from._impl_.bitplane_encoding_);
      } else {
        _this->_impl_.bitplane_encoding_->MergeFrom(*from._impl_.bitplane_encoding_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      ABSL_DCHECK(from._impl_.encoding_ != nullptr);
      if (_this->_impl_.encoding_ == nullptr) {
        _this->_impl_.encoding_ = Super_::CopyConstruct(arena, *from._impl_.encoding_);
      } else {
        _this->_impl_.encoding_->MergeFrom(*from._impl_.encoding_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000400U)) {
      ABSL_DCHECK(from._impl_.device_ != nullptr);
      if (_this->_impl_.device_ == nullptr) {
        _this->_impl_.device_ = Super_::CopyConstruct(arena, *from._impl_.device_);
      } else {
        _this->_impl_.device_->MergeFrom(*from._impl_.device_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Header::CopyFrom(const Header& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:mgard.pb.Header)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Header::InternalSwap(Header* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Header, _impl_.device_)
      + sizeof(Header::_impl_.device_)
      - PROTOBUF_FIELD_OFFSET(Header, _impl_.mgard_version_)>(
          reinterpret_cast<char*>(&_impl_.mgard_version_),
          reinterpret_cast<char*>(&other->_impl_.mgard_version_));
}

::google::protobuf::Metadata Header::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace pb
}  // namespace mgard
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_mgard_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
